package org.openxma.dsl.pom.util;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.EcoreUtil2;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.HeightProperty;
import org.openxma.dsl.pom.model.IAttachmentPosition;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.Offset;
import org.openxma.dsl.pom.model.ParentAttachment;
import org.openxma.dsl.pom.model.StylePropertyCharacterHeight;
import org.openxma.dsl.pom.model.StylePropertyCharacterWidth;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.WidthProperty;

public class DimensionCalculator {
    
    public static final int DEFAULT_CHARACTER_WIDTH = 7;
    public static final int DEFAULT_CHARACTER_HEIGHT = 14;
    
    public static int calculateWidthDimension(EObject obj, int dimensionInChar, int dimensionInPixel, boolean pixelNegative) {
        int dimension;
        if (dimensionInChar != 0) {
            if (!(obj instanceof GuiElement)) {
                obj = EcoreUtil2.getContainerOfType(obj,GuiElement.class);
            }
            Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(obj);             
            StylePropertyCharacterWidth characterWidth = StyleAccess.getCharacterWidthStyle(styleProperties);
            if (characterWidth != null) {
                dimension=dimensionInChar * characterWidth.getCharWidth();
            } else {
                dimension=dimensionInChar * DEFAULT_CHARACTER_WIDTH;
            }
        } else {
            dimension=0;
        }
        if (pixelNegative) {
            dimension-=dimensionInPixel;
        } else {
            dimension+=dimensionInPixel;
        }
        return dimension;
    }
    
    private static int calculateHeightDimension(EObject obj, int dimensionInChar, int dimensionInPixel, boolean pixelNegative) {
        int dimension;
        if ( dimensionInChar != 0 ) {
            if (!(obj instanceof GuiElement)) {
                obj = EcoreUtil2.getContainerOfType(obj,GuiElement.class);
            }
            Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(obj);             
            StylePropertyCharacterHeight characterHeight = StyleAccess.getCharacterHeightStyle(styleProperties);
            if (characterHeight != null) {
                dimension = dimensionInChar * characterHeight.getCharHeight();
            } else {
                dimension = dimensionInChar * DEFAULT_CHARACTER_HEIGHT;
            }
            
        } else {
            dimension = 0;
        }
        if ( pixelNegative ) {
            dimension -= dimensionInPixel;
        } else {
            dimension += dimensionInPixel;
        }
        return dimension;
    }

    public static int calculateWidth(EObject obj, WidthProperty widthProperty) {
        if (widthProperty != null) {
            return calculateWidthDimension(obj, widthProperty.getWidthInChar(),widthProperty.getWidth(),widthProperty.isNegative());
        } else {
            return 0;
        }
    }
    
    public static int calculateHeight(EObject obj, HeightProperty heightProperty) {
        if ( heightProperty != null )
            return calculateHeightDimension(obj,heightProperty.getHeightInChar(),heightProperty.getHeight(),heightProperty.isNegative());
        else
            return 0;
    }
    
    public static int calculateOffset(EObject obj, Offset offset) {
        if (offset != null) {
            int signedCharValue = offset.isNegativeChar() ? offset.getValueInChar() *-1 : offset.getValueInChar();
            return calculateWidthDimension(obj,signedCharValue,offset.getSignedValue(),false);
        } else {
            return 0;
        }        
    }
    
    public static int calculateAttachmentOffset(ParentAttachment parentAttachment) {        
        int attachmentOffset;        
        if (parentAttachment.isRelative()) {
            attachmentOffset = calculateOffset(parentAttachment,parentAttachment.getOffset());
        } else {
            attachmentOffset = calculateWidthDimension(parentAttachment,parentAttachment.getDistanceInChar(),parentAttachment.getDistance(),parentAttachment.isNegative());
        }                        
        return attachmentOffset;
    }
    
    public static int calculateTabulatorOffset(IAttachmentPosition tab) {
        if (tab.isRelative()) {
            return calculateOffset(tab,tab.getOffset());
        } else {            
            return calculateWidthDimension(tab,tab.getWidthInChar(),tab.getWidth(),tab.isNegativeAfterChar());
        }
    }

}
