/**
 * <copyright>
 * </copyright>
 *
 * $Id: PomPackageImpl.java 9623 2012-06-18 09:06:31Z hoenninger $
 */
package org.openxma.dsl.pom.model.impl;

import static org.openxma.dsl.pom.PomPackage.PROXY;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.impl.EPackageImpl;
import org.openxma.dsl.core.CorePackage;
import org.openxma.dsl.dom.DomPackage;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.AbsoluteSiblingAttachment;
import org.openxma.dsl.pom.model.AlignmentProperty;
import org.openxma.dsl.pom.model.AssignStatement;
import org.openxma.dsl.pom.model.AttachmentOwnPosition;
import org.openxma.dsl.pom.model.AttachmentPosition;
import org.openxma.dsl.pom.model.AttachmentProperty;
import org.openxma.dsl.pom.model.AttachmentSiblingPosition;
import org.openxma.dsl.pom.model.AttachmentSpecification;
import org.openxma.dsl.pom.model.BrowseControlType;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.CallStatement;
import org.openxma.dsl.pom.model.CenterProperty;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.CollapsedFlag;
import org.openxma.dsl.pom.model.ColumnInfo;
import org.openxma.dsl.pom.model.ColumnOrder;
import org.openxma.dsl.pom.model.CombinedModelElement;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.CommandImplementation;
import org.openxma.dsl.pom.model.ComplexElement;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.ComposedElement;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.ConditionedLogic;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.ContentAlignment;
import org.openxma.dsl.pom.model.ControlEventMapping;
import org.openxma.dsl.pom.model.ControlType;
import org.openxma.dsl.pom.model.CustomizationOfComposite;
import org.openxma.dsl.pom.model.CustomizationOfGuiElement;
import org.openxma.dsl.pom.model.CustomizationOfTabFolder;
import org.openxma.dsl.pom.model.CustomizeComponentModel;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataBindingElement;
import org.openxma.dsl.pom.model.DataMapping;
import org.openxma.dsl.pom.model.DataMappingList;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.Definitions;
import org.openxma.dsl.pom.model.DropDownMenu;
import org.openxma.dsl.pom.model.EditableFlag;
import org.openxma.dsl.pom.model.EnabledFlag;
import org.openxma.dsl.pom.model.EnterEventMapping;
import org.openxma.dsl.pom.model.EventFunction;
import org.openxma.dsl.pom.model.EventMapping;
import org.openxma.dsl.pom.model.EventMappingList;
import org.openxma.dsl.pom.model.EventType;
import org.openxma.dsl.pom.model.ExecuteStatement;
import org.openxma.dsl.pom.model.FieldFeature;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.FieldReference;
import org.openxma.dsl.pom.model.FieldVariable;
import org.openxma.dsl.pom.model.GrayLogic;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.GuiElementEventMapping;
import org.openxma.dsl.pom.model.GuiElementWithEvent_dummy;
import org.openxma.dsl.pom.model.HeightProperty;
import org.openxma.dsl.pom.model.HorizontalAlignment;
import org.openxma.dsl.pom.model.IAttachmentPosition;
import org.openxma.dsl.pom.model.IComposite;
import org.openxma.dsl.pom.model.IElementOnWhichCanBeAttached;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.IField;
import org.openxma.dsl.pom.model.IGuiElementWithEvent;
import org.openxma.dsl.pom.model.IReferenceableByStatementVariable;
import org.openxma.dsl.pom.model.ITabFolder;
import org.openxma.dsl.pom.model.ITabPage;
import org.openxma.dsl.pom.model.Image;
import org.openxma.dsl.pom.model.ImageProperty;
import org.openxma.dsl.pom.model.ImageUri;
import org.openxma.dsl.pom.model.IncludePanel;
import org.openxma.dsl.pom.model.InitEventMapping;
import org.openxma.dsl.pom.model.IntegerDefinition;
import org.openxma.dsl.pom.model.InvokeStatement;
import org.openxma.dsl.pom.model.Invokeable;
import org.openxma.dsl.pom.model.JoinDirection;
import org.openxma.dsl.pom.model.Label;
import org.openxma.dsl.pom.model.LabelProperty;
import org.openxma.dsl.pom.model.LabelText;
import org.openxma.dsl.pom.model.LayoutDataProperty;
import org.openxma.dsl.pom.model.LeaveEventMapping;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.LogicBlock;
import org.openxma.dsl.pom.model.LogicItem;
import org.openxma.dsl.pom.model.MandatoryFlag;
import org.openxma.dsl.pom.model.Menu;
import org.openxma.dsl.pom.model.MenuItem;
import org.openxma.dsl.pom.model.ModalityProperty;
import org.openxma.dsl.pom.model.ModelElementSpecification;
import org.openxma.dsl.pom.model.MultiselectionFlag;
import org.openxma.dsl.pom.model.NoneAttachment;
import org.openxma.dsl.pom.model.ObjectProperty;
import org.openxma.dsl.pom.model.Offset;
import org.openxma.dsl.pom.model.Orientation;
import org.openxma.dsl.pom.model.PaddingWidth;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.PageCustomization;
import org.openxma.dsl.pom.model.PageDefinition;
import org.openxma.dsl.pom.model.PageProperty;
import org.openxma.dsl.pom.model.PagingControl;
import org.openxma.dsl.pom.model.ParentAttachment;
import org.openxma.dsl.pom.model.PredefinedCommand;
import org.openxma.dsl.pom.model.PropertyJoin;
import org.openxma.dsl.pom.model.Proxy;
import org.openxma.dsl.pom.model.RadioButton;
import org.openxma.dsl.pom.model.ReferencedComposite;
import org.openxma.dsl.pom.model.ReferencedTabFolder;
import org.openxma.dsl.pom.model.ReferencedTabPage;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.ReferencedXMAGuiElement;
import org.openxma.dsl.pom.model.ReferencedXMAPage;
import org.openxma.dsl.pom.model.RelativeSibling;
import org.openxma.dsl.pom.model.RelativeSiblingAttachment;
import org.openxma.dsl.pom.model.ResizeableProperty;
import org.openxma.dsl.pom.model.SelectedModelElement;
import org.openxma.dsl.pom.model.Seperator;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.SiblingAttachment;
import org.openxma.dsl.pom.model.SimpleElement;
import org.openxma.dsl.pom.model.SplitPanel;
import org.openxma.dsl.pom.model.Statement;
import org.openxma.dsl.pom.model.StatementVariable;
import org.openxma.dsl.pom.model.StatusBarProperty;
import org.openxma.dsl.pom.model.StrictFlag;
import org.openxma.dsl.pom.model.StringDefinition;
import org.openxma.dsl.pom.model.StyleProperty;
import org.openxma.dsl.pom.model.StylePropertyAlignment;
import org.openxma.dsl.pom.model.StylePropertyBackgroundColor;
import org.openxma.dsl.pom.model.StylePropertyBorder;
import org.openxma.dsl.pom.model.StylePropertyBottom;
import org.openxma.dsl.pom.model.StylePropertyBrowseControl;
import org.openxma.dsl.pom.model.StylePropertyCharacterHeight;
import org.openxma.dsl.pom.model.StylePropertyCharacterWidth;
import org.openxma.dsl.pom.model.StylePropertyEnumerationDisplayType;
import org.openxma.dsl.pom.model.StylePropertyEnumerationSortOrder;
import org.openxma.dsl.pom.model.StylePropertyFieldPart;
import org.openxma.dsl.pom.model.StylePropertyFlag;
import org.openxma.dsl.pom.model.StylePropertyFont;
import org.openxma.dsl.pom.model.StylePropertyForegroundColor;
import org.openxma.dsl.pom.model.StylePropertyHeight;
import org.openxma.dsl.pom.model.StylePropertyImage;
import org.openxma.dsl.pom.model.StylePropertyLeft;
import org.openxma.dsl.pom.model.StylePropertyLineWrap;
import org.openxma.dsl.pom.model.StylePropertyMultiLine;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StylePropertyPaging;
import org.openxma.dsl.pom.model.StylePropertyPagingControls;
import org.openxma.dsl.pom.model.StylePropertyPagingCustomizerImage;
import org.openxma.dsl.pom.model.StylePropertyPagingJumpSize;
import org.openxma.dsl.pom.model.StylePropertyPagingPageSize;
import org.openxma.dsl.pom.model.StylePropertyPagingPosition;
import org.openxma.dsl.pom.model.StylePropertyRight;
import org.openxma.dsl.pom.model.StylePropertyScrollable;
import org.openxma.dsl.pom.model.StylePropertyTable;
import org.openxma.dsl.pom.model.StylePropertyTableCustomizer;
import org.openxma.dsl.pom.model.StylePropertyTabulator;
import org.openxma.dsl.pom.model.StylePropertyTop;
import org.openxma.dsl.pom.model.StylePropertyWidgetVariant;
import org.openxma.dsl.pom.model.StylePropertyWidth;
import org.openxma.dsl.pom.model.StyleSelectorType;
import org.openxma.dsl.pom.model.StyleSpecification;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabAlignment;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabMenu;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.TabSetting;
import org.openxma.dsl.pom.model.TabableFlag;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.TableCustomizer;
import org.openxma.dsl.pom.model.TabulatorAttachment;
import org.openxma.dsl.pom.model.TabulatorDefinition;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.TextProperty;
import org.openxma.dsl.pom.model.TitleButton;
import org.openxma.dsl.pom.model.TitleButtonsProperty;
import org.openxma.dsl.pom.model.TooltipProperty;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.model.TreeMenu;
import org.openxma.dsl.pom.model.UnitProperty;
import org.openxma.dsl.pom.model.VariableAssignment;
import org.openxma.dsl.pom.model.VariableValue;
import org.openxma.dsl.pom.model.VerticalAlignment;
import org.openxma.dsl.pom.model.VisibleFlag;
import org.openxma.dsl.pom.model.WidthProperty;
import org.openxma.dsl.pom.model.XMAWidgetEventMapping;
import org.openxma.dsl.pom.model.XMAWidgetGrayLogic;
import org.openxma.dsl.pom.model.XmaVariable;
import org.openxma.dsl.pom.model.XmadslComposite;
import org.openxma.dsl.pom.model.XmadslPage;
import org.openxma.dsl.pom.model.*;

import at.spardat.xma.guidesign.GuidesignPackage;
import at.spardat.xma.guidesign.flex.FlexPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model <b>Package</b>.
 * <!-- end-user-doc -->
 * @generated
 */
public class PomPackageImpl extends EPackageImpl implements PomPackage {
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataBindingElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass enabledFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass mandatoryFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass visibleFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass collapsedFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass editableFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass tabableFlagEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass multiselectionFlagEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass strictFlagEClass = null;

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass labelTextEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass objectPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass componentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedXMAGuiElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedXMAPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedXMACompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass grayLogicEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmaWidgetGrayLogicEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataObjectVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass textPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass labelPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tooltipPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass unitPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iFieldEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldFeatureEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldReferenceEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizeableFieldEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldPartSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass commandEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass eventFunctionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass eventMappingListEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass eventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass controlEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmaWidgetEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass guiElementEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass initEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass enterEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass leaveEventMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataMappingListEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass dataMappingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pageDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmadslPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass menuEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass dropDownMenuEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass tabMenuEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass treeMenuEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass menuItemEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass logicBlockEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass conditionedLogicEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass logicItemEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass columnOrderEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass tabSettingEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass columnInfoEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pageCustomizationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass guiElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass compositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass composedElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass complexElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass simpleElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iGuiElementWithEventEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass guiElementWithEvent_dummyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass textEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass comboEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass checkBoxEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass listBoxEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass includePanelEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass treeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass radioButtonEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass seperatorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass labelEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass imageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass buttonEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmadslCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass splitPanelEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass tableComboEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tableColumnEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iTabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedTabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iTabPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass referencedTabPageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass layoutDataPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pagePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass heightPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass widthPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass attachmentPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass modalityPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass imagePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass titleButtonsPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass resizeablePropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass statusBarPropertyEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass centerPropertyEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass alignmentPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass composeDataEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass paddingWidthEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iAttachmentPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabulatorDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stringDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass integerDefinitionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass imageUriEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass definitionsEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabulatorPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass attachmentPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass contentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass setOfGuiElementsEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass attachmentSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass noneAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass parentAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iElementOnWhichCanBeAttachedEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass siblingAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass absoluteSiblingAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass relativeSiblingAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tabulatorAttachmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass offsetEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass fieldVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass xmaVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizeComponentModelEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizationOfGuiElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizationOfCompositeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass customizationOfTabFolderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iElementWithLayoutDataEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass styleSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass modelElementSpecificationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass selectedModelElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass combinedModelElementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass styleSpecificationPropertyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyFieldPartEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyLeftEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyRightEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyTopEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyBottomEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyWidthEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyHeightEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyTabulatorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyBorderEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyForegroundColorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyBackgroundColorEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyWidgetVariantEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyImageEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyAlignmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyTableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyFontEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyCharacterWidthEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyCharacterHeightEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyFlagEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyLineWrapEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyMultiLineEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyBrowseControlEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyPagingEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyPagingControlsEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyPagingPageSizeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyPagingJumpSizeEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyPagingPositionEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyPagingCustomizerImageEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass pagingControlEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass stylePropertyTableCustomizerEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyScrollableEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyEnumerationDisplayTypeEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyEnumerationSortOrderEClass = null;

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass stylePropertyOtherEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass tableCustomizerEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass commandImplementationEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass statementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass statementVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass iReferenceableByStatementVariableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass assignStatementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass variableAssignmentEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass variableValueEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass proxyEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass propertyJoinEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass invokeableEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass executeStatementEClass = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EClass invokeStatementEClass = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EClass callStatementEClass = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum eventTypeEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum predefinedCommandEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum attachmentOwnPositionEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum relativeSiblingEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum attachmentSiblingPositionEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum contentAlignmentEEnum = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EEnum tabAlignmentEEnum = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum horizontalAlignmentEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum verticalAlignmentEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum orientationEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum controlTypeEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum fieldPartEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum titleButtonEEnum = null;

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    private EEnum joinDirectionEEnum = null;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum styleSelectorTypeEEnum = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private EEnum browseControlTypeEEnum = null;

	/**
	 * Creates an instance of the model <b>Package</b>, registered with
	 * {@link org.eclipse.emf.ecore.EPackage.Registry EPackage.Registry} by the package
	 * package URI value.
	 * <p>Note: the correct way to create the package is via the static
	 * factory method {@link #init init()}, which also performs
	 * initialization of the package, or returns the registered package,
	 * if one already exists.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see org.eclipse.emf.ecore.EPackage.Registry
	 * @see org.openxma.dsl.pom.PomPackage#eNS_URI
	 * @see #init()
	 * @generated
	 */
	private PomPackageImpl() {
		super(eNS_URI, PomFactory.eINSTANCE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private static boolean isInited = false;

	/**
	 * Creates, registers, and initializes the <b>Package</b> for this model, and for any others upon which it depends.
	 * 
	 * <p>This method is used to initialize {@link PomPackage#eINSTANCE} when that field is accessed.
	 * Clients should not invoke it directly. Instead, they should simply access that field to obtain the package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #eNS_URI
	 * @see #createPackageContents()
	 * @see #initializePackageContents()
	 * @generated
	 */
	public static PomPackage init() {
		if (isInited) return (PomPackage)EPackage.Registry.INSTANCE.getEPackage(PomPackage.eNS_URI);

		// Obtain or create and register package
		PomPackageImpl thePomPackage = (PomPackageImpl)(EPackage.Registry.INSTANCE.get(eNS_URI) instanceof PomPackageImpl ? EPackage.Registry.INSTANCE.get(eNS_URI) : new PomPackageImpl());

		isInited = true;

		// Initialize simple dependencies
		DomPackage.eINSTANCE.eClass();
		GuidesignPackage.eINSTANCE.eClass();

		// Create package meta-data objects
		thePomPackage.createPackageContents();

		// Initialize created meta-data
		thePomPackage.initializePackageContents();

		// Mark meta-data to indicate it can't be changed
		thePomPackage.freeze();

  
		// Update the registry and return the package
		EPackage.Registry.INSTANCE.put(PomPackage.eNS_URI, thePomPackage);
		return thePomPackage;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataBindingElement() {
		return dataBindingElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldFlag() {
		return fieldFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getFieldFlag_Expression() {
		return (EReference)fieldFlagEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEnabledFlag() {
		return enabledFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getMandatoryFlag() {
		return mandatoryFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getVisibleFlag() {
		return visibleFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCollapsedFlag() {
		return collapsedFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getCollapsedFlag_AvailableSpecified() {
		return (EAttribute)collapsedFlagEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEditableFlag() {
		return editableFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getTabableFlag() {
		return tabableFlagEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTabableFlag_Value() {
		return (EReference)tabableFlagEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getMultiselectionFlag() {
		return multiselectionFlagEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getMultiselectionFlag_MultiSelection() {
		return (EReference)multiselectionFlagEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStrictFlag() {
		return strictFlagEClass;
	}

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStrictFlag_Strict() {
		return (EReference)strictFlagEClass.getEStructuralFeatures().get(0);
	}

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLabelText() {
		return labelTextEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabelText_Label() {
		return (EAttribute)labelTextEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabelText_ReferencedLabelString() {
		return (EReference)labelTextEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getObjectProperty() {
		return objectPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getObjectProperty_Public() {
		return (EAttribute)objectPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getObjectProperty_Type() {
		return (EReference)objectPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getObjectProperty_Name() {
		return (EAttribute)objectPropertyEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getObjectProperty_DefaultValue() {
		return (EAttribute)objectPropertyEClass.getEStructuralFeatures().get(3);
	}

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getObjectProperty_ReadOnly() {
		return (EReference)objectPropertyEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getObjectProperty_Required() {
		return (EReference)objectPropertyEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComponent() {
		return componentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_XmaComponent() {
		return (EReference)componentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_StyleProperty() {
		return (EReference)componentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getComponent_DefaultPage() {
		return (EReference)componentEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Dependencies() {
		return (EReference)componentEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_ComponentProperties() {
		return (EReference)componentEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Dataobjects() {
		return (EReference)componentEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Commands() {
		return (EReference)componentEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Events() {
		return (EReference)componentEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_ConditionsBlock() {
		return (EReference)componentEClass.getEStructuralFeatures().get(8);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Pages() {
		return (EReference)componentEClass.getEStructuralFeatures().get(9);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComponent_Definitions() {
		return (EReference)componentEClass.getEStructuralFeatures().get(10);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedXMAGuiElement() {
		return referencedXMAGuiElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedXMAPage() {
		return referencedXMAPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_XmaPage() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_FieldFlags() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_Dependencies() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getReferencedXMAPage_LogicBlock() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAPage_GrayLogic() {
		return (EReference)referencedXMAPageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedXMAComposite() {
		return referencedXMACompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getReferencedXMAComposite_ReferencingPageComposite() {
		return (EAttribute)referencedXMACompositeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getReferencedXMAComposite_VerticalPartitioning() {
		return (EAttribute)referencedXMACompositeEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getReferencedXMAComposite_HorizontalPartitioning() {
		return (EAttribute)referencedXMACompositeEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAComposite_XmaComposite() {
		return (EReference)referencedXMACompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedXMAComposite_FieldFlags() {
		return (EReference)referencedXMACompositeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getReferencedXMAComposite_TabableFlag() {
		return (EReference)referencedXMACompositeEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGrayLogic() {
		return grayLogicEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getGrayLogic_WidgetGrayLogic() {
		return (EReference)grayLogicEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXMAWidgetGrayLogic() {
		return xmaWidgetGrayLogicEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXMAWidgetGrayLogic_XmaWidget() {
		return (EReference)xmaWidgetGrayLogicEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXMAWidgetGrayLogic_FieldFlags() {
		return (EReference)xmaWidgetGrayLogicEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataObjectVariable() {
		return dataObjectVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataObjectVariable_Type() {
		return (EReference)dataObjectVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getDataObjectVariable_IsCollection() {
		return (EAttribute)dataObjectVariableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getDataObjectVariable_Name() {
		return (EAttribute)dataObjectVariableEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTextProperty() {
		return textPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLabelProperty() {
		return labelPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabelProperty_LabelText() {
		return (EAttribute)labelPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabelProperty_ReferencedLabelString() {
		return (EReference)labelPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTooltipProperty() {
		return tooltipPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTooltipProperty_TooltipText() {
		return (EAttribute)tooltipPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTooltipProperty_ReferencedTooltipString() {
		return (EReference)tooltipPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getUnitProperty() {
		return unitPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getUnitProperty_UnitLabelText() {
		return (EAttribute)unitPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getUnitProperty_ReferencedUnitLabelString() {
		return (EReference)unitPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIField() {
		return iFieldEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIField_Object() {
		return (EReference)iFieldEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIField_Feature() {
		return (EReference)iFieldEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldFeature() {
		return fieldFeatureEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldFeature_Feature() {
		return (EReference)fieldFeatureEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldFeature_SubFeature() {
		return (EReference)fieldFeatureEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldReference() {
		return fieldReferenceEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizeableField() {
		return customizeableFieldEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCustomizeableField_ControlType() {
		return (EAttribute)customizeableFieldEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_Parts() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_MultiLine() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_Constraints() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_Format() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeableField_TextProperties() {
		return (EReference)customizeableFieldEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldPartSpecification() {
		return fieldPartSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getFieldPartSpecification_Part() {
		return (EAttribute)fieldPartSpecificationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldPartSpecification_LayoutProperties() {
		return (EReference)fieldPartSpecificationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCommand() {
		return commandEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCommand_Doc() {
		return (EAttribute)commandEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCommand_ClientOnly() {
		return (EAttribute)commandEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCommand_Name() {
		return (EAttribute)commandEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCommand_Implementation() {
		return (EReference)commandEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEventFunction() {
		return eventFunctionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getEventFunction_Command() {
		return (EReference)eventFunctionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getEventFunction_PredefinedCommand() {
		return (EAttribute)eventFunctionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEventMappingList() {
		return eventMappingListEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getEventMappingList_Mapping() {
		return (EReference)eventMappingListEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEventMapping() {
		return eventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getEventMapping_EventFunctions() {
		return (EReference)eventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getControlEventMapping() {
		return controlEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getControlEventMapping_Event() {
		return (EAttribute)controlEventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXMAWidgetEventMapping() {
		return xmaWidgetEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXMAWidgetEventMapping_Control() {
		return (EReference)xmaWidgetEventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGuiElementEventMapping() {
		return guiElementEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getGuiElementEventMapping_Control() {
		return (EReference)guiElementEventMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getInitEventMapping() {
		return initEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getEnterEventMapping() {
		return enterEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLeaveEventMapping() {
		return leaveEventMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataMappingList() {
		return dataMappingListEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMappingList_Mapping() {
		return (EReference)dataMappingListEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDataMapping() {
		return dataMappingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMapping_Control() {
		return (EReference)dataMappingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMapping_BindingElement() {
		return (EReference)dataMappingEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDataMapping_Field() {
		return (EReference)dataMappingEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPageDefinition() {
		return pageDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPage() {
		return pageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_ComposeLayout() {
		return (EReference)pageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Dataobjects() {
		return (EReference)pageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_DataMapping() {
		return (EReference)pageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Commands() {
		return (EReference)pageEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Events() {
		return (EReference)pageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_ConditionsBlock() {
		return (EReference)pageEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPage_Definitions() {
		return (EReference)pageEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXmadslPage() {
		return xmadslPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslPage_Abstract() {
		return (EAttribute)xmadslPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslPage_Name() {
		return (EAttribute)xmadslPageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_Template() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_Dependencies() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_PageProperties() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getXmadslPage_Menus() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(7);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getXmadslPage_LogicBlock() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(8);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_Extensions() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(9);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getMenu() {
		return menuEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getMenu_MenuItems() {
		return (EReference)menuEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getDropDownMenu() {
		return dropDownMenuEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getTabMenu() {
		return tabMenuEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTabMenu_Panel() {
		return (EReference)tabMenuEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getTabMenu_Alignment() {
		return (EAttribute)tabMenuEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabMenu_Defaultselection() {
		return (EReference)tabMenuEClass.getEStructuralFeatures().get(2);
	}

				/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getTreeMenu() {
		return treeMenuEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTreeMenu_Panel() {
		return (EReference)treeMenuEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getMenuItem() {
		return menuItemEClass;
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getMenuItem_Name() {
		return (EAttribute)menuItemEClass.getEStructuralFeatures().get(0);
	}

				/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getMenuItem_Label() {
		return (EAttribute)menuItemEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getMenuItem_Accelerator() {
		return (EAttribute)menuItemEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getMenuItem_Image() {
		return (EAttribute)menuItemEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getMenuItem_Defaultselection() {
		return (EReference)menuItemEClass.getEStructuralFeatures().get(4);
	}

				/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getMenuItem_Enabled() {
		return (EReference)menuItemEClass.getEStructuralFeatures().get(5);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getMenuItem_MenuItem() {
		return (EReference)menuItemEClass.getEStructuralFeatures().get(6);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getLogicBlock() {
		return logicBlockEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicBlock_LogicItem() {
		return (EReference)logicBlockEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicBlock_ConditionedLogic() {
		return (EReference)logicBlockEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getConditionedLogic() {
		return conditionedLogicEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getConditionedLogic_StatusFlag() {
		return (EReference)conditionedLogicEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getConditionedLogic_Default() {
		return (EAttribute)conditionedLogicEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getConditionedLogic_LogicItem() {
		return (EReference)conditionedLogicEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getLogicItem() {
		return logicItemEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicItem_GuiElement() {
		return (EReference)logicItemEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicItem_FieldFlags() {
		return (EReference)logicItemEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicItem_TabableFlag() {
		return (EReference)logicItemEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicItem_ColumnOrder() {
		return (EReference)logicItemEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getLogicItem_TabSettings() {
		return (EReference)logicItemEClass.getEStructuralFeatures().get(4);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getColumnOrder() {
		return columnOrderEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getColumnOrder_ColumnInfos() {
		return (EReference)columnOrderEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getTabSetting() {
		return tabSettingEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTabSetting_OrderedElements() {
		return (EReference)tabSettingEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getColumnInfo() {
		return columnInfoEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getColumnInfo_Columns() {
		return (EReference)columnInfoEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getColumnInfo_Width() {
		return (EAttribute)columnInfoEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getColumnInfo_RelativeWidth() {
		return (EAttribute)columnInfoEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getColumnInfo_LabelText() {
		return (EReference)columnInfoEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_LabelText() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslPage_PageProperty() {
		return (EReference)xmadslPageEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPageCustomization() {
		return pageCustomizationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPageCustomization_PageToCustomize() {
		return (EReference)pageCustomizationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPageCustomization_Composites() {
		return (EReference)pageCustomizationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGuiElement() {
		return guiElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComposite() {
		return compositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposite_Content() {
		return (EReference)compositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComposedElement() {
		return composedElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComplexElement() {
		return complexElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSimpleElement() {
		return simpleElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIGuiElementWithEvent() {
		return iGuiElementWithEventEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getGuiElementWithEvent_dummy() {
		return guiElementWithEvent_dummyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getGuiElementWithEvent_dummy_DummyProperty() {
		return (EAttribute)guiElementWithEvent_dummyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getText() {
		return textEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getText_Name() {
		return (EAttribute)textEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getText_Alignment() {
		return (EAttribute)textEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_MultiLine() {
		return (EReference)textEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_Constraints() {
		return (EReference)textEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_Format() {
		return (EReference)textEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_Type() {
		return (EReference)textEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getText_TextProperties() {
		return (EReference)textEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCombo() {
		return comboEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCombo_Name() {
		return (EAttribute)comboEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_Constraints() {
		return (EReference)comboEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_Format() {
		return (EReference)comboEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_Type() {
		return (EReference)comboEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_StrictFlag() {
		return (EReference)comboEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombo_TextProperties() {
		return (EReference)comboEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCheckBox() {
		return checkBoxEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCheckBox_Name() {
		return (EAttribute)checkBoxEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_LabelText() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_Constraints() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_Format() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCheckBox_TextProperties() {
		return (EReference)checkBoxEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getListBox() {
		return listBoxEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getListBox_Name() {
		return (EAttribute)listBoxEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getListBox_MultiselectionFlag() {
		return (EReference)listBoxEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getListBox_TextProperties() {
		return (EReference)listBoxEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getIncludePanel() {
		return includePanelEClass;
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIncludePanel_Name() {
		return (EAttribute)includePanelEClass.getEStructuralFeatures().get(0);
	}

				/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getIncludePanel_XmaComponent() {
		return (EReference)includePanelEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIncludePanel_Component() {
		return (EReference)includePanelEClass.getEStructuralFeatures().get(2);
	}

				/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getIncludePanel_EmbeddedPage() {
		return (EReference)includePanelEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIncludePanel_XmadslPage() {
		return (EReference)includePanelEClass.getEStructuralFeatures().get(4);
	}

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTree() {
		return treeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTree_Name() {
		return (EAttribute)treeEClass.getEStructuralFeatures().get(0);
	}

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTree_TextProperties() {
		return (EReference)treeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getRadioButton() {
		return radioButtonEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getRadioButton_Name() {
		return (EAttribute)radioButtonEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_LabelText() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_Constraints() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_Format() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getRadioButton_TextProperties() {
		return (EReference)radioButtonEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSeperator() {
		return seperatorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSeperator_Name() {
		return (EAttribute)seperatorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSeperator_Orientation() {
		return (EAttribute)seperatorEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLabel() {
		return labelEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabel_Name() {
		return (EAttribute)labelEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabel_LabelText() {
		return (EReference)labelEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabel_ImageUri() {
		return (EAttribute)labelEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getLabel_ImageUriReference() {
		return (EReference)labelEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getLabel_Alignment() {
		return (EAttribute)labelEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getImage() {
		return imageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImage_Name() {
		return (EAttribute)imageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImage_ImageUri() {
		return (EAttribute)imageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getImage_ImageUriReference() {
		return (EReference)imageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getButton() {
		return buttonEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getButton_Name() {
		return (EAttribute)buttonEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getButton_LabelText() {
		return (EReference)buttonEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getButton_ImageUri() {
		return (EAttribute)buttonEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getButton_ImageUriReference() {
		return (EReference)buttonEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getButton_Alignment() {
		return (EAttribute)buttonEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIComposite() {
		return iCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXmadslComposite() {
		return xmadslCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslComposite_Name() {
		return (EAttribute)xmadslCompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslComposite_Scrollable() {
		return (EReference)xmadslCompositeEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmadslComposite_LabelText() {
		return (EReference)xmadslCompositeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslComposite_VerticalPartitioning() {
		return (EAttribute)xmadslCompositeEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSplitPanel() {
		return splitPanelEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSplitPanel_SplitPosition() {
		return (EAttribute)splitPanelEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSplitPanel_Orientation() {
		return (EAttribute)splitPanelEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSplitPanel_FirstPanel() {
		return (EReference)splitPanelEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSplitPanel_SecondPanel() {
		return (EReference)splitPanelEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmadslComposite_HorizontalPartitioning() {
		return (EAttribute)xmadslCompositeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedComposite() {
		return referencedCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedComposite_Reference() {
		return (EReference)referencedCompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getTableCombo() {
		return tableComboEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getTableCombo_Name() {
		return (EAttribute)tableComboEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTableCombo_Object() {
		return (EReference)tableComboEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTableCombo_Key() {
		return (EReference)tableComboEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTableCombo_Columns() {
		return (EReference)tableComboEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTable() {
		return tableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTable_Name() {
		return (EAttribute)tableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Object() {
		return (EReference)tableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Key() {
		return (EReference)tableEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTable_ColumnMinWidth() {
		return (EAttribute)tableEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTable_MultiselectionFlag() {
		return (EReference)tableEClass.getEStructuralFeatures().get(4);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Oneway() {
		return (EReference)tableEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Gridvisible() {
		return (EReference)tableEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Sortindicator() {
		return (EReference)tableEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Pageable() {
		return (EReference)tableEClass.getEStructuralFeatures().get(8);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Filterable() {
		return (EReference)tableEClass.getEStructuralFeatures().get(9);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_CheckSelection() {
		return (EReference)tableEClass.getEStructuralFeatures().get(10);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Collapsed() {
		return (EReference)tableEClass.getEStructuralFeatures().get(11);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTable_Columns() {
		return (EReference)tableEClass.getEStructuralFeatures().get(12);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTableColumn() {
		return tableColumnEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_Object() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_Feature() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_ColumnName() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_Visible() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTableColumn_Label() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTableColumn_StyleProperty() {
		return (EReference)tableColumnEClass.getEStructuralFeatures().get(5);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_Align() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_Width() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_RelativeWidth() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(8);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_MinWidth() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(9);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableColumn_MaxWidth() {
		return (EAttribute)tableColumnEClass.getEStructuralFeatures().get(10);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getITabFolder() {
		return iTabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getITabFolder_TabItems() {
		return (EReference)iTabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabFolder() {
		return tabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabFolder_Name() {
		return (EAttribute)tabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedTabFolder() {
		return referencedTabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedTabFolder_Reference() {
		return (EReference)referencedTabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getITabPage() {
		return iTabPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabPage() {
		return tabPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabPage_Name() {
		return (EAttribute)tabPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_LabelText() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_Scrollable() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTabPage_StyleProperty() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_FieldFlags() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTabPage_TabableFlag() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(5);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabPage_PageProperties() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getTabPage_LogicBlock() {
		return (EReference)tabPageEClass.getEStructuralFeatures().get(7);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getReferencedTabPage() {
		return referencedTabPageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getReferencedTabPage_Reference() {
		return (EReference)referencedTabPageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getLayoutDataProperty() {
		return layoutDataPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPageProperty() {
		return pagePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getHeightProperty() {
		return heightPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getHeightProperty_HeightInChar() {
		return (EAttribute)heightPropertyEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getHeightProperty_Negative() {
		return (EAttribute)heightPropertyEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getHeightProperty_Height() {
		return (EAttribute)heightPropertyEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getWidthProperty() {
		return widthPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getWidthProperty_WidthInChar() {
		return (EAttribute)widthPropertyEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getWidthProperty_Negative() {
		return (EAttribute)widthPropertyEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getWidthProperty_Width() {
		return (EAttribute)widthPropertyEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAttachmentProperty() {
		return attachmentPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAttachmentProperty_OwnPosition() {
		return (EAttribute)attachmentPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getAttachmentProperty_AttachmentSpecification() {
		return (EReference)attachmentPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStyleProperty() {
		return stylePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleProperty_Styles() {
		return (EReference)stylePropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getModalityProperty() {
		return modalityPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getModalityProperty_Modality() {
		return (EAttribute)modalityPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getImageProperty() {
		return imagePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImageProperty_ImageUri() {
		return (EAttribute)imagePropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getImageProperty_ImageUriReference() {
		return (EReference)imagePropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTitleButtonsProperty() {
		return titleButtonsPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTitleButtonsProperty_Button() {
		return (EAttribute)titleButtonsPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getResizeableProperty() {
		return resizeablePropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getResizeableProperty_Resizeable() {
		return (EAttribute)resizeablePropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStatusBarProperty() {
		return statusBarPropertyEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStatusBarProperty_StatusBar() {
		return (EAttribute)statusBarPropertyEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getCenterProperty() {
		return centerPropertyEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getCenterProperty_Center() {
		return (EAttribute)centerPropertyEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAlignmentProperty() {
		return alignmentPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAlignmentProperty_HAlignment() {
		return (EAttribute)alignmentPropertyEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAlignmentProperty_VAlignment() {
		return (EAttribute)alignmentPropertyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getComposeData() {
		return composeDataEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_Tabulators() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_ReferencedTabulator() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_CellWidths() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getComposeData_CellWidthsCount() {
		return (EAttribute)composeDataEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_CellHeights() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_Spacing() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getComposeData_Margin() {
		return (EReference)composeDataEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPaddingWidth() {
		return paddingWidthEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getPaddingWidth_Width() {
		return (EAttribute)paddingWidthEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIAttachmentPosition() {
		return iAttachmentPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIAttachmentPosition_Name() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIAttachmentPosition_Width() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getIAttachmentPosition_WidthInChar() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getIAttachmentPosition_NegativeAfterChar() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIAttachmentPosition_Relative() {
		return (EAttribute)iAttachmentPositionEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIAttachmentPosition_Offset() {
		return (EReference)iAttachmentPositionEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIAttachmentPosition_ReferencedPosition() {
		return (EReference)iAttachmentPositionEClass.getEStructuralFeatures().get(6);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIAttachmentPosition_AdditionalOffset() {
		return (EReference)iAttachmentPositionEClass.getEStructuralFeatures().get(7);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabulatorDefinition() {
		return tabulatorDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorDefinition_Name() {
		return (EAttribute)tabulatorDefinitionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabulatorDefinition_Tabulators() {
		return (EReference)tabulatorDefinitionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStringDefinition() {
		return stringDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStringDefinition_Name() {
		return (EAttribute)stringDefinitionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStringDefinition_Text() {
		return (EAttribute)stringDefinitionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIntegerDefinition() {
		return integerDefinitionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIntegerDefinition_Name() {
		return (EAttribute)integerDefinitionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getIntegerDefinition_Value() {
		return (EAttribute)integerDefinitionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getImageUri() {
		return imageUriEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImageUri_Name() {
		return (EAttribute)imageUriEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getImageUri_Source() {
		return (EAttribute)imageUriEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getDefinitions() {
		return definitionsEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Strings() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Integers() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_ImageSources() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Positions() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getDefinitions_Tabulators() {
		return (EReference)definitionsEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabulatorPosition() {
		return tabulatorPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorPosition_FloatingPosition() {
		return (EAttribute)tabulatorPositionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorPosition_FillingPosition() {
		return (EAttribute)tabulatorPositionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorPosition_DeltaSpecification() {
		return (EAttribute)tabulatorPositionEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAttachmentPosition() {
		return attachmentPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getContent() {
		return contentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_WidgetSetItems() {
		return (EReference)contentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_ContainerItems() {
		return (EReference)contentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_Elements() {
		return (EReference)contentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getContent_ComposeLayout() {
		return (EReference)contentEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSetOfGuiElements() {
		return setOfGuiElementsEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getSetOfGuiElements_FieldFlags() {
		return (EReference)setOfGuiElementsEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getSetOfGuiElements_TabableFlag() {
		return (EReference)setOfGuiElementsEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSetOfGuiElements_Elements() {
		return (EReference)setOfGuiElementsEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAttachmentSpecification() {
		return attachmentSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getNoneAttachment() {
		return noneAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getNoneAttachment_None() {
		return (EAttribute)noneAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getParentAttachment() {
		return parentAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getParentAttachment_DistanceInChar() {
		return (EAttribute)parentAttachmentEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getParentAttachment_Negative() {
		return (EAttribute)parentAttachmentEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getParentAttachment_Distance() {
		return (EAttribute)parentAttachmentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getParentAttachment_Relative() {
		return (EAttribute)parentAttachmentEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getParentAttachment_Offset() {
		return (EReference)parentAttachmentEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIElementOnWhichCanBeAttached() {
		return iElementOnWhichCanBeAttachedEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSiblingAttachment() {
		return siblingAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSiblingAttachment_SiblingPosition() {
		return (EAttribute)siblingAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSiblingAttachment_Offset() {
		return (EReference)siblingAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAbsoluteSiblingAttachment() {
		return absoluteSiblingAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getAbsoluteSiblingAttachment_Sibling() {
		return (EReference)absoluteSiblingAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getAbsoluteSiblingAttachment_FieldPart() {
		return (EAttribute)absoluteSiblingAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getRelativeSiblingAttachment() {
		return relativeSiblingAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getRelativeSiblingAttachment_RelativeSibling() {
		return (EAttribute)relativeSiblingAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTabulatorAttachment() {
		return tabulatorAttachmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTabulatorAttachment_TabIndex() {
		return (EAttribute)tabulatorAttachmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabulatorAttachment_Sibling() {
		return (EReference)tabulatorAttachmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getTabulatorAttachment_Offset() {
		return (EReference)tabulatorAttachmentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getOffset() {
		return offsetEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getOffset_NegativeChar() {
		return (EAttribute)offsetEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getOffset_ValueInChar() {
		return (EAttribute)offsetEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getOffset_Negative() {
		return (EAttribute)offsetEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getOffset_Value() {
		return (EAttribute)offsetEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getFieldVariable() {
		return fieldVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getFieldVariable_Field() {
		return (EReference)fieldVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getFieldVariable_Access() {
		return (EAttribute)fieldVariableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getXmaVariable() {
		return xmaVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getXmaVariable_Reference() {
		return (EReference)xmaVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getXmaVariable_Access() {
		return (EAttribute)xmaVariableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizeComponentModel() {
		return customizeComponentModelEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeComponentModel_Imports() {
		return (EReference)customizeComponentModelEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeComponentModel_Component() {
		return (EReference)customizeComponentModelEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizeComponentModel_Customizations() {
		return (EReference)customizeComponentModelEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizationOfGuiElement() {
		return customizationOfGuiElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfGuiElement_Page() {
		return (EReference)customizationOfGuiElementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizationOfComposite() {
		return customizationOfCompositeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_Composite() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_ComposeLayout() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_AddedElements() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCustomizationOfComposite_VerticalPartitioning() {
		return (EAttribute)customizationOfCompositeEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCustomizationOfComposite_HorizontalPartitioning() {
		return (EAttribute)customizationOfCompositeEClass.getEStructuralFeatures().get(4);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfComposite_AddedWidgetSetItems() {
		return (EReference)customizationOfCompositeEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCustomizationOfTabFolder() {
		return customizationOfTabFolderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfTabFolder_TabFolder() {
		return (EReference)customizationOfTabFolderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCustomizationOfTabFolder_AddedTabItems() {
		return (EReference)customizationOfTabFolderEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIElementWithLayoutData() {
		return iElementWithLayoutDataEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIElementWithLayoutData_LayoutProperties() {
		return (EReference)iElementWithLayoutDataEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIElementWithLayoutData_FieldFlags() {
		return (EReference)iElementWithLayoutDataEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getIElementWithLayoutData_TabableFlag() {
		return (EReference)iElementWithLayoutDataEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStyleSpecification() {
		return styleSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleSpecification_ModelElements() {
		return (EReference)styleSpecificationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleSpecification_SuperStyles() {
		return (EReference)styleSpecificationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStyleSpecification_StyleProperty() {
		return (EReference)styleSpecificationEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getModelElementSpecification() {
		return modelElementSpecificationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getModelElementSpecification_ModelElement() {
		return (EReference)modelElementSpecificationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getModelElementSpecification_CombinedElements() {
		return (EReference)modelElementSpecificationEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getSelectedModelElement() {
		return selectedModelElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_Type() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getSelectedModelElement_SimpleType() {
		return (EReference)selectedModelElementEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_Name() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getSelectedModelElement_PseudoClass() {
		return (EAttribute)selectedModelElementEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCombinedModelElement() {
		return combinedModelElementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCombinedModelElement_Child() {
		return (EAttribute)combinedModelElementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getCombinedModelElement_Next() {
		return (EAttribute)combinedModelElementEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCombinedModelElement_ModelElement() {
		return (EReference)combinedModelElementEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStyleSpecificationProperty() {
		return styleSpecificationPropertyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyFieldPart() {
		return stylePropertyFieldPartEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFieldPart_Parts() {
		return (EReference)stylePropertyFieldPartEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyLeft() {
		return stylePropertyLeftEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyLeft_AttachmentSpecification() {
		return (EReference)stylePropertyLeftEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyRight() {
		return stylePropertyRightEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyRight_AttachmentSpecification() {
		return (EReference)stylePropertyRightEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyTop() {
		return stylePropertyTopEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTop_AttachmentSpecification() {
		return (EReference)stylePropertyTopEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyBottom() {
		return stylePropertyBottomEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyBottom_AttachmentSpecification() {
		return (EReference)stylePropertyBottomEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyWidth() {
		return stylePropertyWidthEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyWidth_Width() {
		return (EAttribute)stylePropertyWidthEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyWidth_WidthInChar() {
		return (EAttribute)stylePropertyWidthEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyWidth_NegativeAfterChar() {
		return (EAttribute)stylePropertyWidthEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyHeight() {
		return stylePropertyHeightEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyHeight_Height() {
		return (EAttribute)stylePropertyHeightEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyTabulator() {
		return stylePropertyTabulatorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTabulator_Tabulators() {
		return (EReference)stylePropertyTabulatorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyBorder() {
		return stylePropertyBorderEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBorder_NoneBorder() {
		return (EAttribute)stylePropertyBorderEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBorder_Solid() {
		return (EAttribute)stylePropertyBorderEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyForegroundColor() {
		return stylePropertyForegroundColorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_Red() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_Blue() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_Green() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyForegroundColor_ColorID() {
		return (EAttribute)stylePropertyForegroundColorEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyBackgroundColor() {
		return stylePropertyBackgroundColorEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_Red() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_Blue() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_Green() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyBackgroundColor_ColorID() {
		return (EAttribute)stylePropertyBackgroundColorEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyWidgetVariant() {
		return stylePropertyWidgetVariantEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyWidgetVariant_Name() {
		return (EAttribute)stylePropertyWidgetVariantEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyImage() {
		return stylePropertyImageEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyImage_ImageUri() {
		return (EAttribute)stylePropertyImageEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyImage_ImageUriReference() {
		return (EReference)stylePropertyImageEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyAlignment() {
		return stylePropertyAlignmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyAlignment_Alignment() {
		return (EAttribute)stylePropertyAlignmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyTable() {
		return stylePropertyTableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTable_OneWay() {
		return (EReference)stylePropertyTableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTable_GridVisible() {
		return (EReference)stylePropertyTableEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyFont() {
		return stylePropertyFontEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyFont_Styles() {
		return (EAttribute)stylePropertyFontEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyFont_Size() {
		return (EAttribute)stylePropertyFontEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyFont_Name() {
		return (EAttribute)stylePropertyFontEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyCharacterWidth() {
		return stylePropertyCharacterWidthEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyCharacterWidth_CharWidth() {
		return (EAttribute)stylePropertyCharacterWidthEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyCharacterHeight() {
		return stylePropertyCharacterHeightEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyCharacterHeight_CharHeight() {
		return (EAttribute)stylePropertyCharacterHeightEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyFlag() {
		return stylePropertyFlagEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFlag_EnabledExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFlag_MandatoryExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFlag_VisibleExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFlag_CollapsedExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyFlag_AvailableExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(4);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyFlag_EditableExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyFlag_TabableExpression() {
		return (EReference)stylePropertyFlagEClass.getEStructuralFeatures().get(6);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyLineWrap() {
		return stylePropertyLineWrapEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyLineWrap_Wrap() {
		return (EReference)stylePropertyLineWrapEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyMultiLine() {
		return stylePropertyMultiLineEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyMultiLine_Multi() {
		return (EReference)stylePropertyMultiLineEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyBrowseControl() {
		return stylePropertyBrowseControlEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyBrowseControl_ControlType() {
		return (EAttribute)stylePropertyBrowseControlEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyPaging() {
		return stylePropertyPagingEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPaging_Style() {
		return (EAttribute)stylePropertyPagingEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPaging_Default() {
		return (EAttribute)stylePropertyPagingEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyPagingControls() {
		return stylePropertyPagingControlsEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyPagingControls_Controls() {
		return (EReference)stylePropertyPagingControlsEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyPagingPageSize() {
		return stylePropertyPagingPageSizeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPagingPageSize_Size() {
		return (EAttribute)stylePropertyPagingPageSizeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPagingPageSize_MaxSize() {
		return (EAttribute)stylePropertyPagingPageSizeEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyPagingJumpSize() {
		return stylePropertyPagingJumpSizeEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPagingJumpSize_Size() {
		return (EAttribute)stylePropertyPagingJumpSizeEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyPagingPosition() {
		return stylePropertyPagingPositionEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPagingPosition_Top() {
		return (EAttribute)stylePropertyPagingPositionEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyPagingPosition_Bottom() {
		return (EAttribute)stylePropertyPagingPositionEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyPagingCustomizerImage() {
		return stylePropertyPagingCustomizerImageEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyPagingCustomizerImage_ImageUri() {
		return (EAttribute)stylePropertyPagingCustomizerImageEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyPagingCustomizerImage_ImageUriReference() {
		return (EReference)stylePropertyPagingCustomizerImageEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPagingControl() {
		return pagingControlEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getPagingControl_Control() {
		return (EAttribute)pagingControlEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getPagingControl_End() {
		return (EAttribute)pagingControlEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getPagingControl_Next() {
		return (EAttribute)pagingControlEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getPagingControl_Customize() {
		return (EAttribute)pagingControlEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getPagingControl_Export() {
		return (EAttribute)pagingControlEClass.getEStructuralFeatures().get(4);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStylePropertyTableCustomizer() {
		return stylePropertyTableCustomizerEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStylePropertyTableCustomizer_TableCustomizer() {
		return (EReference)stylePropertyTableCustomizerEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyScrollable() {
		return stylePropertyScrollableEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyScrollable_NotScrollable() {
		return (EAttribute)stylePropertyScrollableEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyScrollable_Orientations() {
		return (EAttribute)stylePropertyScrollableEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyEnumerationDisplayType() {
		return stylePropertyEnumerationDisplayTypeEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyEnumerationDisplayType_Type() {
		return (EAttribute)stylePropertyEnumerationDisplayTypeEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyEnumerationSortOrder() {
		return stylePropertyEnumerationSortOrderEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getStylePropertyEnumerationSortOrder_Sortorder() {
		return (EAttribute)stylePropertyEnumerationSortOrderEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getStylePropertyOther() {
		return stylePropertyOtherEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyOther_DynamicTabPage() {
		return (EReference)stylePropertyOtherEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyOther_ThousandSeperator() {
		return (EReference)stylePropertyOtherEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyOther_Spacing() {
		return (EReference)stylePropertyOtherEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getStylePropertyOther_Margin() {
		return (EReference)stylePropertyOtherEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getStylePropertyOther_WidgetEffects() {
		return (EAttribute)stylePropertyOtherEClass.getEStructuralFeatures().get(4);
	}

				/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getTableCustomizer() {
		return tableCustomizerEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getTableCustomizer_InstanceType() {
		return (EAttribute)tableCustomizerEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getCommandImplementation() {
		return commandImplementationEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getCommandImplementation_Statements() {
		return (EReference)commandImplementationEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStatement() {
		return statementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getStatementVariable() {
		return statementVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getStatementVariable_Reference() {
		return (EReference)statementVariableEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getIReferenceableByStatementVariable() {
		return iReferenceableByStatementVariableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getAssignStatement() {
		return assignStatementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getAssignStatement_VariableAssignments() {
		return (EReference)assignStatementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getVariableAssignment() {
		return variableAssignmentEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getVariableAssignment_Left() {
		return (EReference)variableAssignmentEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getVariableAssignment_Value() {
		return (EReference)variableAssignmentEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getVariableAssignment_ServiceCall() {
		return (EReference)variableAssignmentEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getVariableValue() {
		return variableValueEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getVariableValue_Variable() {
		return (EReference)variableValueEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getVariableValue_Access() {
		return (EAttribute)variableValueEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getVariableValue_StringLiteral() {
		return (EAttribute)variableValueEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EAttribute getVariableValue_IntegerLiteral() {
		return (EAttribute)variableValueEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getProxy() {
		return proxyEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getProxy_Url() {
		return (EAttribute)proxyEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getProxy_Properties() {
		return (EReference)proxyEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getPropertyJoin() {
		return propertyJoinEClass;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getPropertyJoin_Variable() {
		return (EReference)propertyJoinEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getPropertyJoin_Access() {
		return (EAttribute)propertyJoinEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getPropertyJoin_StringLiteral() {
		return (EAttribute)propertyJoinEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getPropertyJoin_IntegerLiteral() {
		return (EAttribute)propertyJoinEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getPropertyJoin_JoinDirection() {
		return (EAttribute)propertyJoinEClass.getEStructuralFeatures().get(4);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getPropertyJoin_Property() {
		return (EReference)propertyJoinEClass.getEStructuralFeatures().get(5);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getInvokeable() {
		return invokeableEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getExecuteStatement() {
		return executeStatementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getExecuteStatement_Service() {
		return (EReference)executeStatementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getExecuteStatement_Operation() {
		return (EReference)executeStatementEClass.getEStructuralFeatures().get(1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getExecuteStatement_DelegateOperation() {
		return (EReference)executeStatementEClass.getEStructuralFeatures().get(2);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getExecuteStatement_ParameterValues() {
		return (EReference)executeStatementEClass.getEStructuralFeatures().get(3);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EClass getInvokeStatement() {
		return invokeStatementEClass;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EReference getInvokeStatement_InvokeAble() {
		return (EReference)invokeStatementEClass.getEStructuralFeatures().get(0);
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getInvokeStatement_TargetPanel() {
		return (EReference)invokeStatementEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getInvokeStatement_PropertyJoin() {
		return (EReference)invokeStatementEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EClass getCallStatement() {
		return callStatementEClass;
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getCallStatement_Command() {
		return (EReference)callStatementEClass.getEStructuralFeatures().get(0);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getCallStatement_Left() {
		return (EReference)callStatementEClass.getEStructuralFeatures().get(1);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EAttribute getCallStatement_JavaCommand() {
		return (EAttribute)callStatementEClass.getEStructuralFeatures().get(2);
	}

    /**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EReference getCallStatement_ParameterValues() {
		return (EReference)callStatementEClass.getEStructuralFeatures().get(3);
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getEventType() {
		return eventTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getPredefinedCommand() {
		return predefinedCommandEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getAttachmentOwnPosition() {
		return attachmentOwnPositionEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getRelativeSibling() {
		return relativeSiblingEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getAttachmentSiblingPosition() {
		return attachmentSiblingPositionEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getContentAlignment() {
		return contentAlignmentEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EEnum getTabAlignment() {
		return tabAlignmentEEnum;
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getHorizontalAlignment() {
		return horizontalAlignmentEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getVerticalAlignment() {
		return verticalAlignmentEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getOrientation() {
		return orientationEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getControlType() {
		return controlTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getFieldPart() {
		return fieldPartEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getTitleButton() {
		return titleButtonEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
	 * @generated
	 */
    public EEnum getJoinDirection() {
		return joinDirectionEEnum;
	}

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getStyleSelectorType() {
		return styleSelectorTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EEnum getBrowseControlType() {
		return browseControlTypeEEnum;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public PomFactory getPomFactory() {
		return (PomFactory)getEFactoryInstance();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private boolean isCreated = false;

	/**
	 * Creates the meta-model objects for the package.  This method is
	 * guarded to have no affect on any invocation but its first.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void createPackageContents() {
		if (isCreated) return;
		isCreated = true;

		// Create classes and their features
		iGuiElementWithEventEClass = createEClass(IGUI_ELEMENT_WITH_EVENT);

		iElementWithLayoutDataEClass = createEClass(IELEMENT_WITH_LAYOUT_DATA);
		createEReference(iElementWithLayoutDataEClass, IELEMENT_WITH_LAYOUT_DATA__LAYOUT_PROPERTIES);
		createEReference(iElementWithLayoutDataEClass, IELEMENT_WITH_LAYOUT_DATA__FIELD_FLAGS);
		createEReference(iElementWithLayoutDataEClass, IELEMENT_WITH_LAYOUT_DATA__TABABLE_FLAG);

		dataBindingElementEClass = createEClass(DATA_BINDING_ELEMENT);

		fieldFlagEClass = createEClass(FIELD_FLAG);
		createEReference(fieldFlagEClass, FIELD_FLAG__EXPRESSION);

		enabledFlagEClass = createEClass(ENABLED_FLAG);

		mandatoryFlagEClass = createEClass(MANDATORY_FLAG);

		visibleFlagEClass = createEClass(VISIBLE_FLAG);

		collapsedFlagEClass = createEClass(COLLAPSED_FLAG);
		createEAttribute(collapsedFlagEClass, COLLAPSED_FLAG__AVAILABLE_SPECIFIED);

		editableFlagEClass = createEClass(EDITABLE_FLAG);

		tabableFlagEClass = createEClass(TABABLE_FLAG);
		createEReference(tabableFlagEClass, TABABLE_FLAG__VALUE);

		multiselectionFlagEClass = createEClass(MULTISELECTION_FLAG);
		createEReference(multiselectionFlagEClass, MULTISELECTION_FLAG__MULTI_SELECTION);

		strictFlagEClass = createEClass(STRICT_FLAG);
		createEReference(strictFlagEClass, STRICT_FLAG__STRICT);

		labelTextEClass = createEClass(LABEL_TEXT);
		createEAttribute(labelTextEClass, LABEL_TEXT__LABEL);
		createEReference(labelTextEClass, LABEL_TEXT__REFERENCED_LABEL_STRING);

		objectPropertyEClass = createEClass(OBJECT_PROPERTY);
		createEAttribute(objectPropertyEClass, OBJECT_PROPERTY__PUBLIC);
		createEReference(objectPropertyEClass, OBJECT_PROPERTY__TYPE);
		createEAttribute(objectPropertyEClass, OBJECT_PROPERTY__NAME);
		createEAttribute(objectPropertyEClass, OBJECT_PROPERTY__DEFAULT_VALUE);
		createEReference(objectPropertyEClass, OBJECT_PROPERTY__READ_ONLY);
		createEReference(objectPropertyEClass, OBJECT_PROPERTY__REQUIRED);

		componentEClass = createEClass(COMPONENT);
		createEReference(componentEClass, COMPONENT__XMA_COMPONENT);
		createEReference(componentEClass, COMPONENT__STYLE_PROPERTY);
		createEReference(componentEClass, COMPONENT__DEFAULT_PAGE);
		createEReference(componentEClass, COMPONENT__DEPENDENCIES);
		createEReference(componentEClass, COMPONENT__COMPONENT_PROPERTIES);
		createEReference(componentEClass, COMPONENT__DATAOBJECTS);
		createEReference(componentEClass, COMPONENT__COMMANDS);
		createEReference(componentEClass, COMPONENT__EVENTS);
		createEReference(componentEClass, COMPONENT__CONDITIONS_BLOCK);
		createEReference(componentEClass, COMPONENT__PAGES);
		createEReference(componentEClass, COMPONENT__DEFINITIONS);

		referencedXMAGuiElementEClass = createEClass(REFERENCED_XMA_GUI_ELEMENT);

		referencedXMAPageEClass = createEClass(REFERENCED_XMA_PAGE);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__XMA_PAGE);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__FIELD_FLAGS);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__DEPENDENCIES);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__LOGIC_BLOCK);
		createEReference(referencedXMAPageEClass, REFERENCED_XMA_PAGE__GRAY_LOGIC);

		referencedXMACompositeEClass = createEClass(REFERENCED_XMA_COMPOSITE);
		createEReference(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__XMA_COMPOSITE);
		createEReference(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__FIELD_FLAGS);
		createEReference(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__TABABLE_FLAG);
		createEAttribute(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__REFERENCING_PAGE_COMPOSITE);
		createEAttribute(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__HORIZONTAL_PARTITIONING);
		createEAttribute(referencedXMACompositeEClass, REFERENCED_XMA_COMPOSITE__VERTICAL_PARTITIONING);

		grayLogicEClass = createEClass(GRAY_LOGIC);
		createEReference(grayLogicEClass, GRAY_LOGIC__WIDGET_GRAY_LOGIC);

		xmaWidgetGrayLogicEClass = createEClass(XMA_WIDGET_GRAY_LOGIC);
		createEReference(xmaWidgetGrayLogicEClass, XMA_WIDGET_GRAY_LOGIC__XMA_WIDGET);
		createEReference(xmaWidgetGrayLogicEClass, XMA_WIDGET_GRAY_LOGIC__FIELD_FLAGS);

		dataObjectVariableEClass = createEClass(DATA_OBJECT_VARIABLE);
		createEReference(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__TYPE);
		createEAttribute(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__IS_COLLECTION);
		createEAttribute(dataObjectVariableEClass, DATA_OBJECT_VARIABLE__NAME);

		textPropertyEClass = createEClass(TEXT_PROPERTY);

		labelPropertyEClass = createEClass(LABEL_PROPERTY);
		createEAttribute(labelPropertyEClass, LABEL_PROPERTY__LABEL_TEXT);
		createEReference(labelPropertyEClass, LABEL_PROPERTY__REFERENCED_LABEL_STRING);

		tooltipPropertyEClass = createEClass(TOOLTIP_PROPERTY);
		createEAttribute(tooltipPropertyEClass, TOOLTIP_PROPERTY__TOOLTIP_TEXT);
		createEReference(tooltipPropertyEClass, TOOLTIP_PROPERTY__REFERENCED_TOOLTIP_STRING);

		unitPropertyEClass = createEClass(UNIT_PROPERTY);
		createEAttribute(unitPropertyEClass, UNIT_PROPERTY__UNIT_LABEL_TEXT);
		createEReference(unitPropertyEClass, UNIT_PROPERTY__REFERENCED_UNIT_LABEL_STRING);

		iFieldEClass = createEClass(IFIELD);
		createEReference(iFieldEClass, IFIELD__OBJECT);
		createEReference(iFieldEClass, IFIELD__FEATURE);

		fieldFeatureEClass = createEClass(FIELD_FEATURE);
		createEReference(fieldFeatureEClass, FIELD_FEATURE__FEATURE);
		createEReference(fieldFeatureEClass, FIELD_FEATURE__SUB_FEATURE);

		fieldReferenceEClass = createEClass(FIELD_REFERENCE);

		customizeableFieldEClass = createEClass(CUSTOMIZEABLE_FIELD);
		createEAttribute(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__CONTROL_TYPE);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__PARTS);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__MULTI_LINE);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__CONSTRAINTS);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__FORMAT);
		createEReference(customizeableFieldEClass, CUSTOMIZEABLE_FIELD__TEXT_PROPERTIES);

		fieldPartSpecificationEClass = createEClass(FIELD_PART_SPECIFICATION);
		createEAttribute(fieldPartSpecificationEClass, FIELD_PART_SPECIFICATION__PART);
		createEReference(fieldPartSpecificationEClass, FIELD_PART_SPECIFICATION__LAYOUT_PROPERTIES);

		commandEClass = createEClass(COMMAND);
		createEAttribute(commandEClass, COMMAND__DOC);
		createEAttribute(commandEClass, COMMAND__CLIENT_ONLY);
		createEAttribute(commandEClass, COMMAND__NAME);
		createEReference(commandEClass, COMMAND__IMPLEMENTATION);

		eventFunctionEClass = createEClass(EVENT_FUNCTION);
		createEReference(eventFunctionEClass, EVENT_FUNCTION__COMMAND);
		createEAttribute(eventFunctionEClass, EVENT_FUNCTION__PREDEFINED_COMMAND);

		eventMappingListEClass = createEClass(EVENT_MAPPING_LIST);
		createEReference(eventMappingListEClass, EVENT_MAPPING_LIST__MAPPING);

		eventMappingEClass = createEClass(EVENT_MAPPING);
		createEReference(eventMappingEClass, EVENT_MAPPING__EVENT_FUNCTIONS);

		controlEventMappingEClass = createEClass(CONTROL_EVENT_MAPPING);
		createEAttribute(controlEventMappingEClass, CONTROL_EVENT_MAPPING__EVENT);

		xmaWidgetEventMappingEClass = createEClass(XMA_WIDGET_EVENT_MAPPING);
		createEReference(xmaWidgetEventMappingEClass, XMA_WIDGET_EVENT_MAPPING__CONTROL);

		guiElementEventMappingEClass = createEClass(GUI_ELEMENT_EVENT_MAPPING);
		createEReference(guiElementEventMappingEClass, GUI_ELEMENT_EVENT_MAPPING__CONTROL);

		initEventMappingEClass = createEClass(INIT_EVENT_MAPPING);

		enterEventMappingEClass = createEClass(ENTER_EVENT_MAPPING);

		leaveEventMappingEClass = createEClass(LEAVE_EVENT_MAPPING);

		dataMappingListEClass = createEClass(DATA_MAPPING_LIST);
		createEReference(dataMappingListEClass, DATA_MAPPING_LIST__MAPPING);

		dataMappingEClass = createEClass(DATA_MAPPING);
		createEReference(dataMappingEClass, DATA_MAPPING__CONTROL);
		createEReference(dataMappingEClass, DATA_MAPPING__BINDING_ELEMENT);
		createEReference(dataMappingEClass, DATA_MAPPING__FIELD);

		pageDefinitionEClass = createEClass(PAGE_DEFINITION);

		pageEClass = createEClass(PAGE);
		createEReference(pageEClass, PAGE__COMPOSE_LAYOUT);
		createEReference(pageEClass, PAGE__DATAOBJECTS);
		createEReference(pageEClass, PAGE__DATA_MAPPING);
		createEReference(pageEClass, PAGE__COMMANDS);
		createEReference(pageEClass, PAGE__EVENTS);
		createEReference(pageEClass, PAGE__CONDITIONS_BLOCK);
		createEReference(pageEClass, PAGE__DEFINITIONS);

		xmadslPageEClass = createEClass(XMADSL_PAGE);
		createEAttribute(xmadslPageEClass, XMADSL_PAGE__ABSTRACT);
		createEAttribute(xmadslPageEClass, XMADSL_PAGE__NAME);
		createEReference(xmadslPageEClass, XMADSL_PAGE__LABEL_TEXT);
		createEReference(xmadslPageEClass, XMADSL_PAGE__PAGE_PROPERTY);
		createEReference(xmadslPageEClass, XMADSL_PAGE__TEMPLATE);
		createEReference(xmadslPageEClass, XMADSL_PAGE__DEPENDENCIES);
		createEReference(xmadslPageEClass, XMADSL_PAGE__PAGE_PROPERTIES);
		createEReference(xmadslPageEClass, XMADSL_PAGE__MENUS);
		createEReference(xmadslPageEClass, XMADSL_PAGE__LOGIC_BLOCK);
		createEReference(xmadslPageEClass, XMADSL_PAGE__EXTENSIONS);

		menuEClass = createEClass(MENU);
		createEReference(menuEClass, MENU__MENU_ITEMS);

		tabMenuEClass = createEClass(TAB_MENU);
		createEReference(tabMenuEClass, TAB_MENU__PANEL);
		createEAttribute(tabMenuEClass, TAB_MENU__ALIGNMENT);
		createEReference(tabMenuEClass, TAB_MENU__DEFAULTSELECTION);

		dropDownMenuEClass = createEClass(DROP_DOWN_MENU);

		treeMenuEClass = createEClass(TREE_MENU);
		createEReference(treeMenuEClass, TREE_MENU__PANEL);

		menuItemEClass = createEClass(MENU_ITEM);
		createEAttribute(menuItemEClass, MENU_ITEM__NAME);
		createEAttribute(menuItemEClass, MENU_ITEM__LABEL);
		createEAttribute(menuItemEClass, MENU_ITEM__ACCELERATOR);
		createEAttribute(menuItemEClass, MENU_ITEM__IMAGE);
		createEReference(menuItemEClass, MENU_ITEM__DEFAULTSELECTION);
		createEReference(menuItemEClass, MENU_ITEM__ENABLED);
		createEReference(menuItemEClass, MENU_ITEM__MENU_ITEM);

		logicBlockEClass = createEClass(LOGIC_BLOCK);
		createEReference(logicBlockEClass, LOGIC_BLOCK__LOGIC_ITEM);
		createEReference(logicBlockEClass, LOGIC_BLOCK__CONDITIONED_LOGIC);

		conditionedLogicEClass = createEClass(CONDITIONED_LOGIC);
		createEReference(conditionedLogicEClass, CONDITIONED_LOGIC__STATUS_FLAG);
		createEAttribute(conditionedLogicEClass, CONDITIONED_LOGIC__DEFAULT);
		createEReference(conditionedLogicEClass, CONDITIONED_LOGIC__LOGIC_ITEM);

		logicItemEClass = createEClass(LOGIC_ITEM);
		createEReference(logicItemEClass, LOGIC_ITEM__GUI_ELEMENT);
		createEReference(logicItemEClass, LOGIC_ITEM__FIELD_FLAGS);
		createEReference(logicItemEClass, LOGIC_ITEM__TABABLE_FLAG);
		createEReference(logicItemEClass, LOGIC_ITEM__COLUMN_ORDER);
		createEReference(logicItemEClass, LOGIC_ITEM__TAB_SETTINGS);

		columnOrderEClass = createEClass(COLUMN_ORDER);
		createEReference(columnOrderEClass, COLUMN_ORDER__COLUMN_INFOS);

		tabSettingEClass = createEClass(TAB_SETTING);
		createEReference(tabSettingEClass, TAB_SETTING__ORDERED_ELEMENTS);

		columnInfoEClass = createEClass(COLUMN_INFO);
		createEReference(columnInfoEClass, COLUMN_INFO__COLUMNS);
		createEAttribute(columnInfoEClass, COLUMN_INFO__WIDTH);
		createEAttribute(columnInfoEClass, COLUMN_INFO__RELATIVE_WIDTH);
		createEReference(columnInfoEClass, COLUMN_INFO__LABEL_TEXT);

		pageCustomizationEClass = createEClass(PAGE_CUSTOMIZATION);
		createEReference(pageCustomizationEClass, PAGE_CUSTOMIZATION__PAGE_TO_CUSTOMIZE);
		createEReference(pageCustomizationEClass, PAGE_CUSTOMIZATION__COMPOSITES);

		guiElementEClass = createEClass(GUI_ELEMENT);

		compositeEClass = createEClass(COMPOSITE);
		createEReference(compositeEClass, COMPOSITE__CONTENT);

		composedElementEClass = createEClass(COMPOSED_ELEMENT);

		complexElementEClass = createEClass(COMPLEX_ELEMENT);

		simpleElementEClass = createEClass(SIMPLE_ELEMENT);

		guiElementWithEvent_dummyEClass = createEClass(GUI_ELEMENT_WITH_EVENT_DUMMY);
		createEAttribute(guiElementWithEvent_dummyEClass, GUI_ELEMENT_WITH_EVENT_DUMMY__DUMMY_PROPERTY);

		textEClass = createEClass(TEXT);
		createEAttribute(textEClass, TEXT__NAME);
		createEAttribute(textEClass, TEXT__ALIGNMENT);
		createEReference(textEClass, TEXT__MULTI_LINE);
		createEReference(textEClass, TEXT__CONSTRAINTS);
		createEReference(textEClass, TEXT__FORMAT);
		createEReference(textEClass, TEXT__TYPE);
		createEReference(textEClass, TEXT__TEXT_PROPERTIES);

		comboEClass = createEClass(COMBO);
		createEAttribute(comboEClass, COMBO__NAME);
		createEReference(comboEClass, COMBO__CONSTRAINTS);
		createEReference(comboEClass, COMBO__FORMAT);
		createEReference(comboEClass, COMBO__TYPE);
		createEReference(comboEClass, COMBO__STRICT_FLAG);
		createEReference(comboEClass, COMBO__TEXT_PROPERTIES);

		checkBoxEClass = createEClass(CHECK_BOX);
		createEAttribute(checkBoxEClass, CHECK_BOX__NAME);
		createEReference(checkBoxEClass, CHECK_BOX__LABEL_TEXT);
		createEReference(checkBoxEClass, CHECK_BOX__CONSTRAINTS);
		createEReference(checkBoxEClass, CHECK_BOX__FORMAT);
		createEReference(checkBoxEClass, CHECK_BOX__TEXT_PROPERTIES);

		listBoxEClass = createEClass(LIST_BOX);
		createEAttribute(listBoxEClass, LIST_BOX__NAME);
		createEReference(listBoxEClass, LIST_BOX__MULTISELECTION_FLAG);
		createEReference(listBoxEClass, LIST_BOX__TEXT_PROPERTIES);

		includePanelEClass = createEClass(INCLUDE_PANEL);
		createEAttribute(includePanelEClass, INCLUDE_PANEL__NAME);
		createEReference(includePanelEClass, INCLUDE_PANEL__XMA_COMPONENT);
		createEReference(includePanelEClass, INCLUDE_PANEL__COMPONENT);
		createEReference(includePanelEClass, INCLUDE_PANEL__EMBEDDED_PAGE);
		createEReference(includePanelEClass, INCLUDE_PANEL__XMADSL_PAGE);

		treeEClass = createEClass(TREE);
		createEAttribute(treeEClass, TREE__NAME);
		createEReference(treeEClass, TREE__TEXT_PROPERTIES);

		radioButtonEClass = createEClass(RADIO_BUTTON);
		createEAttribute(radioButtonEClass, RADIO_BUTTON__NAME);
		createEReference(radioButtonEClass, RADIO_BUTTON__LABEL_TEXT);
		createEReference(radioButtonEClass, RADIO_BUTTON__CONSTRAINTS);
		createEReference(radioButtonEClass, RADIO_BUTTON__FORMAT);
		createEReference(radioButtonEClass, RADIO_BUTTON__TEXT_PROPERTIES);

		seperatorEClass = createEClass(SEPERATOR);
		createEAttribute(seperatorEClass, SEPERATOR__NAME);
		createEAttribute(seperatorEClass, SEPERATOR__ORIENTATION);

		labelEClass = createEClass(LABEL);
		createEAttribute(labelEClass, LABEL__NAME);
		createEReference(labelEClass, LABEL__LABEL_TEXT);
		createEAttribute(labelEClass, LABEL__IMAGE_URI);
		createEReference(labelEClass, LABEL__IMAGE_URI_REFERENCE);
		createEAttribute(labelEClass, LABEL__ALIGNMENT);

		imageEClass = createEClass(IMAGE);
		createEAttribute(imageEClass, IMAGE__NAME);
		createEAttribute(imageEClass, IMAGE__IMAGE_URI);
		createEReference(imageEClass, IMAGE__IMAGE_URI_REFERENCE);

		buttonEClass = createEClass(BUTTON);
		createEAttribute(buttonEClass, BUTTON__NAME);
		createEReference(buttonEClass, BUTTON__LABEL_TEXT);
		createEAttribute(buttonEClass, BUTTON__IMAGE_URI);
		createEReference(buttonEClass, BUTTON__IMAGE_URI_REFERENCE);
		createEAttribute(buttonEClass, BUTTON__ALIGNMENT);

		iCompositeEClass = createEClass(ICOMPOSITE);

		xmadslCompositeEClass = createEClass(XMADSL_COMPOSITE);
		createEAttribute(xmadslCompositeEClass, XMADSL_COMPOSITE__NAME);
		createEReference(xmadslCompositeEClass, XMADSL_COMPOSITE__LABEL_TEXT);
		createEReference(xmadslCompositeEClass, XMADSL_COMPOSITE__SCROLLABLE);
		createEAttribute(xmadslCompositeEClass, XMADSL_COMPOSITE__HORIZONTAL_PARTITIONING);
		createEAttribute(xmadslCompositeEClass, XMADSL_COMPOSITE__VERTICAL_PARTITIONING);

		splitPanelEClass = createEClass(SPLIT_PANEL);
		createEAttribute(splitPanelEClass, SPLIT_PANEL__SPLIT_POSITION);
		createEAttribute(splitPanelEClass, SPLIT_PANEL__ORIENTATION);
		createEReference(splitPanelEClass, SPLIT_PANEL__FIRST_PANEL);
		createEReference(splitPanelEClass, SPLIT_PANEL__SECOND_PANEL);

		referencedCompositeEClass = createEClass(REFERENCED_COMPOSITE);
		createEReference(referencedCompositeEClass, REFERENCED_COMPOSITE__REFERENCE);

		tableComboEClass = createEClass(TABLE_COMBO);
		createEAttribute(tableComboEClass, TABLE_COMBO__NAME);
		createEReference(tableComboEClass, TABLE_COMBO__OBJECT);
		createEReference(tableComboEClass, TABLE_COMBO__KEY);
		createEReference(tableComboEClass, TABLE_COMBO__COLUMNS);

		tableEClass = createEClass(TABLE);
		createEAttribute(tableEClass, TABLE__NAME);
		createEReference(tableEClass, TABLE__OBJECT);
		createEReference(tableEClass, TABLE__KEY);
		createEAttribute(tableEClass, TABLE__COLUMN_MIN_WIDTH);
		createEReference(tableEClass, TABLE__MULTISELECTION_FLAG);
		createEReference(tableEClass, TABLE__ONEWAY);
		createEReference(tableEClass, TABLE__GRIDVISIBLE);
		createEReference(tableEClass, TABLE__SORTINDICATOR);
		createEReference(tableEClass, TABLE__PAGEABLE);
		createEReference(tableEClass, TABLE__FILTERABLE);
		createEReference(tableEClass, TABLE__CHECK_SELECTION);
		createEReference(tableEClass, TABLE__COLLAPSED);
		createEReference(tableEClass, TABLE__COLUMNS);

		tableColumnEClass = createEClass(TABLE_COLUMN);
		createEReference(tableColumnEClass, TABLE_COLUMN__OBJECT);
		createEReference(tableColumnEClass, TABLE_COLUMN__FEATURE);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__COLUMN_NAME);
		createEReference(tableColumnEClass, TABLE_COLUMN__VISIBLE);
		createEReference(tableColumnEClass, TABLE_COLUMN__LABEL);
		createEReference(tableColumnEClass, TABLE_COLUMN__STYLE_PROPERTY);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__ALIGN);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__WIDTH);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__RELATIVE_WIDTH);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__MIN_WIDTH);
		createEAttribute(tableColumnEClass, TABLE_COLUMN__MAX_WIDTH);

		iTabFolderEClass = createEClass(ITAB_FOLDER);
		createEReference(iTabFolderEClass, ITAB_FOLDER__TAB_ITEMS);

		tabFolderEClass = createEClass(TAB_FOLDER);
		createEAttribute(tabFolderEClass, TAB_FOLDER__NAME);

		referencedTabFolderEClass = createEClass(REFERENCED_TAB_FOLDER);
		createEReference(referencedTabFolderEClass, REFERENCED_TAB_FOLDER__REFERENCE);

		iTabPageEClass = createEClass(ITAB_PAGE);

		tabPageEClass = createEClass(TAB_PAGE);
		createEAttribute(tabPageEClass, TAB_PAGE__NAME);
		createEReference(tabPageEClass, TAB_PAGE__LABEL_TEXT);
		createEReference(tabPageEClass, TAB_PAGE__SCROLLABLE);
		createEReference(tabPageEClass, TAB_PAGE__STYLE_PROPERTY);
		createEReference(tabPageEClass, TAB_PAGE__FIELD_FLAGS);
		createEReference(tabPageEClass, TAB_PAGE__TABABLE_FLAG);
		createEReference(tabPageEClass, TAB_PAGE__PAGE_PROPERTIES);
		createEReference(tabPageEClass, TAB_PAGE__LOGIC_BLOCK);

		referencedTabPageEClass = createEClass(REFERENCED_TAB_PAGE);
		createEReference(referencedTabPageEClass, REFERENCED_TAB_PAGE__REFERENCE);

		layoutDataPropertyEClass = createEClass(LAYOUT_DATA_PROPERTY);

		pagePropertyEClass = createEClass(PAGE_PROPERTY);

		heightPropertyEClass = createEClass(HEIGHT_PROPERTY);
		createEAttribute(heightPropertyEClass, HEIGHT_PROPERTY__HEIGHT_IN_CHAR);
		createEAttribute(heightPropertyEClass, HEIGHT_PROPERTY__NEGATIVE);
		createEAttribute(heightPropertyEClass, HEIGHT_PROPERTY__HEIGHT);

		widthPropertyEClass = createEClass(WIDTH_PROPERTY);
		createEAttribute(widthPropertyEClass, WIDTH_PROPERTY__WIDTH_IN_CHAR);
		createEAttribute(widthPropertyEClass, WIDTH_PROPERTY__NEGATIVE);
		createEAttribute(widthPropertyEClass, WIDTH_PROPERTY__WIDTH);

		stylePropertyEClass = createEClass(STYLE_PROPERTY);
		createEReference(stylePropertyEClass, STYLE_PROPERTY__STYLES);

		modalityPropertyEClass = createEClass(MODALITY_PROPERTY);
		createEAttribute(modalityPropertyEClass, MODALITY_PROPERTY__MODALITY);

		imagePropertyEClass = createEClass(IMAGE_PROPERTY);
		createEAttribute(imagePropertyEClass, IMAGE_PROPERTY__IMAGE_URI);
		createEReference(imagePropertyEClass, IMAGE_PROPERTY__IMAGE_URI_REFERENCE);

		titleButtonsPropertyEClass = createEClass(TITLE_BUTTONS_PROPERTY);
		createEAttribute(titleButtonsPropertyEClass, TITLE_BUTTONS_PROPERTY__BUTTON);

		resizeablePropertyEClass = createEClass(RESIZEABLE_PROPERTY);
		createEAttribute(resizeablePropertyEClass, RESIZEABLE_PROPERTY__RESIZEABLE);

		statusBarPropertyEClass = createEClass(STATUS_BAR_PROPERTY);
		createEAttribute(statusBarPropertyEClass, STATUS_BAR_PROPERTY__STATUS_BAR);

		centerPropertyEClass = createEClass(CENTER_PROPERTY);
		createEAttribute(centerPropertyEClass, CENTER_PROPERTY__CENTER);

		alignmentPropertyEClass = createEClass(ALIGNMENT_PROPERTY);
		createEAttribute(alignmentPropertyEClass, ALIGNMENT_PROPERTY__HALIGNMENT);
		createEAttribute(alignmentPropertyEClass, ALIGNMENT_PROPERTY__VALIGNMENT);

		composeDataEClass = createEClass(COMPOSE_DATA);
		createEReference(composeDataEClass, COMPOSE_DATA__TABULATORS);
		createEReference(composeDataEClass, COMPOSE_DATA__REFERENCED_TABULATOR);
		createEReference(composeDataEClass, COMPOSE_DATA__CELL_WIDTHS);
		createEAttribute(composeDataEClass, COMPOSE_DATA__CELL_WIDTHS_COUNT);
		createEReference(composeDataEClass, COMPOSE_DATA__CELL_HEIGHTS);
		createEReference(composeDataEClass, COMPOSE_DATA__SPACING);
		createEReference(composeDataEClass, COMPOSE_DATA__MARGIN);

		paddingWidthEClass = createEClass(PADDING_WIDTH);
		createEAttribute(paddingWidthEClass, PADDING_WIDTH__WIDTH);

		iAttachmentPositionEClass = createEClass(IATTACHMENT_POSITION);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__NAME);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__WIDTH);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__WIDTH_IN_CHAR);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__NEGATIVE_AFTER_CHAR);
		createEAttribute(iAttachmentPositionEClass, IATTACHMENT_POSITION__RELATIVE);
		createEReference(iAttachmentPositionEClass, IATTACHMENT_POSITION__OFFSET);
		createEReference(iAttachmentPositionEClass, IATTACHMENT_POSITION__REFERENCED_POSITION);
		createEReference(iAttachmentPositionEClass, IATTACHMENT_POSITION__ADDITIONAL_OFFSET);

		tabulatorDefinitionEClass = createEClass(TABULATOR_DEFINITION);
		createEAttribute(tabulatorDefinitionEClass, TABULATOR_DEFINITION__NAME);
		createEReference(tabulatorDefinitionEClass, TABULATOR_DEFINITION__TABULATORS);

		stringDefinitionEClass = createEClass(STRING_DEFINITION);
		createEAttribute(stringDefinitionEClass, STRING_DEFINITION__NAME);
		createEAttribute(stringDefinitionEClass, STRING_DEFINITION__TEXT);

		integerDefinitionEClass = createEClass(INTEGER_DEFINITION);
		createEAttribute(integerDefinitionEClass, INTEGER_DEFINITION__NAME);
		createEAttribute(integerDefinitionEClass, INTEGER_DEFINITION__VALUE);

		imageUriEClass = createEClass(IMAGE_URI);
		createEAttribute(imageUriEClass, IMAGE_URI__NAME);
		createEAttribute(imageUriEClass, IMAGE_URI__SOURCE);

		definitionsEClass = createEClass(DEFINITIONS);
		createEReference(definitionsEClass, DEFINITIONS__STRINGS);
		createEReference(definitionsEClass, DEFINITIONS__INTEGERS);
		createEReference(definitionsEClass, DEFINITIONS__IMAGE_SOURCES);
		createEReference(definitionsEClass, DEFINITIONS__POSITIONS);
		createEReference(definitionsEClass, DEFINITIONS__TABULATORS);

		tabulatorPositionEClass = createEClass(TABULATOR_POSITION);
		createEAttribute(tabulatorPositionEClass, TABULATOR_POSITION__FLOATING_POSITION);
		createEAttribute(tabulatorPositionEClass, TABULATOR_POSITION__FILLING_POSITION);
		createEAttribute(tabulatorPositionEClass, TABULATOR_POSITION__DELTA_SPECIFICATION);

		attachmentPositionEClass = createEClass(ATTACHMENT_POSITION);

		contentEClass = createEClass(CONTENT);
		createEReference(contentEClass, CONTENT__WIDGET_SET_ITEMS);
		createEReference(contentEClass, CONTENT__CONTAINER_ITEMS);
		createEReference(contentEClass, CONTENT__ELEMENTS);
		createEReference(contentEClass, CONTENT__COMPOSE_LAYOUT);

		setOfGuiElementsEClass = createEClass(SET_OF_GUI_ELEMENTS);
		createEReference(setOfGuiElementsEClass, SET_OF_GUI_ELEMENTS__FIELD_FLAGS);
		createEReference(setOfGuiElementsEClass, SET_OF_GUI_ELEMENTS__TABABLE_FLAG);
		createEReference(setOfGuiElementsEClass, SET_OF_GUI_ELEMENTS__ELEMENTS);

		attachmentPropertyEClass = createEClass(ATTACHMENT_PROPERTY);
		createEAttribute(attachmentPropertyEClass, ATTACHMENT_PROPERTY__OWN_POSITION);
		createEReference(attachmentPropertyEClass, ATTACHMENT_PROPERTY__ATTACHMENT_SPECIFICATION);

		attachmentSpecificationEClass = createEClass(ATTACHMENT_SPECIFICATION);

		noneAttachmentEClass = createEClass(NONE_ATTACHMENT);
		createEAttribute(noneAttachmentEClass, NONE_ATTACHMENT__NONE);

		parentAttachmentEClass = createEClass(PARENT_ATTACHMENT);
		createEAttribute(parentAttachmentEClass, PARENT_ATTACHMENT__DISTANCE_IN_CHAR);
		createEAttribute(parentAttachmentEClass, PARENT_ATTACHMENT__NEGATIVE);
		createEAttribute(parentAttachmentEClass, PARENT_ATTACHMENT__DISTANCE);
		createEAttribute(parentAttachmentEClass, PARENT_ATTACHMENT__RELATIVE);
		createEReference(parentAttachmentEClass, PARENT_ATTACHMENT__OFFSET);

		iElementOnWhichCanBeAttachedEClass = createEClass(IELEMENT_ON_WHICH_CAN_BE_ATTACHED);

		siblingAttachmentEClass = createEClass(SIBLING_ATTACHMENT);
		createEAttribute(siblingAttachmentEClass, SIBLING_ATTACHMENT__SIBLING_POSITION);
		createEReference(siblingAttachmentEClass, SIBLING_ATTACHMENT__OFFSET);

		absoluteSiblingAttachmentEClass = createEClass(ABSOLUTE_SIBLING_ATTACHMENT);
		createEReference(absoluteSiblingAttachmentEClass, ABSOLUTE_SIBLING_ATTACHMENT__SIBLING);
		createEAttribute(absoluteSiblingAttachmentEClass, ABSOLUTE_SIBLING_ATTACHMENT__FIELD_PART);

		relativeSiblingAttachmentEClass = createEClass(RELATIVE_SIBLING_ATTACHMENT);
		createEAttribute(relativeSiblingAttachmentEClass, RELATIVE_SIBLING_ATTACHMENT__RELATIVE_SIBLING);

		tabulatorAttachmentEClass = createEClass(TABULATOR_ATTACHMENT);
		createEAttribute(tabulatorAttachmentEClass, TABULATOR_ATTACHMENT__TAB_INDEX);
		createEReference(tabulatorAttachmentEClass, TABULATOR_ATTACHMENT__SIBLING);
		createEReference(tabulatorAttachmentEClass, TABULATOR_ATTACHMENT__OFFSET);

		offsetEClass = createEClass(OFFSET);
		createEAttribute(offsetEClass, OFFSET__NEGATIVE_CHAR);
		createEAttribute(offsetEClass, OFFSET__VALUE_IN_CHAR);
		createEAttribute(offsetEClass, OFFSET__NEGATIVE);
		createEAttribute(offsetEClass, OFFSET__VALUE);

		fieldVariableEClass = createEClass(FIELD_VARIABLE);
		createEReference(fieldVariableEClass, FIELD_VARIABLE__FIELD);
		createEAttribute(fieldVariableEClass, FIELD_VARIABLE__ACCESS);

		xmaVariableEClass = createEClass(XMA_VARIABLE);
		createEReference(xmaVariableEClass, XMA_VARIABLE__REFERENCE);
		createEAttribute(xmaVariableEClass, XMA_VARIABLE__ACCESS);

		customizeComponentModelEClass = createEClass(CUSTOMIZE_COMPONENT_MODEL);
		createEReference(customizeComponentModelEClass, CUSTOMIZE_COMPONENT_MODEL__IMPORTS);
		createEReference(customizeComponentModelEClass, CUSTOMIZE_COMPONENT_MODEL__COMPONENT);
		createEReference(customizeComponentModelEClass, CUSTOMIZE_COMPONENT_MODEL__CUSTOMIZATIONS);

		customizationOfGuiElementEClass = createEClass(CUSTOMIZATION_OF_GUI_ELEMENT);
		createEReference(customizationOfGuiElementEClass, CUSTOMIZATION_OF_GUI_ELEMENT__PAGE);

		customizationOfCompositeEClass = createEClass(CUSTOMIZATION_OF_COMPOSITE);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__COMPOSITE);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__COMPOSE_LAYOUT);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__ADDED_ELEMENTS);
		createEAttribute(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__VERTICAL_PARTITIONING);
		createEAttribute(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__HORIZONTAL_PARTITIONING);
		createEReference(customizationOfCompositeEClass, CUSTOMIZATION_OF_COMPOSITE__ADDED_WIDGET_SET_ITEMS);

		customizationOfTabFolderEClass = createEClass(CUSTOMIZATION_OF_TAB_FOLDER);
		createEReference(customizationOfTabFolderEClass, CUSTOMIZATION_OF_TAB_FOLDER__TAB_FOLDER);
		createEReference(customizationOfTabFolderEClass, CUSTOMIZATION_OF_TAB_FOLDER__ADDED_TAB_ITEMS);

		styleSpecificationEClass = createEClass(STYLE_SPECIFICATION);
		createEReference(styleSpecificationEClass, STYLE_SPECIFICATION__MODEL_ELEMENTS);
		createEReference(styleSpecificationEClass, STYLE_SPECIFICATION__SUPER_STYLES);
		createEReference(styleSpecificationEClass, STYLE_SPECIFICATION__STYLE_PROPERTY);

		modelElementSpecificationEClass = createEClass(MODEL_ELEMENT_SPECIFICATION);
		createEReference(modelElementSpecificationEClass, MODEL_ELEMENT_SPECIFICATION__MODEL_ELEMENT);
		createEReference(modelElementSpecificationEClass, MODEL_ELEMENT_SPECIFICATION__COMBINED_ELEMENTS);

		selectedModelElementEClass = createEClass(SELECTED_MODEL_ELEMENT);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__TYPE);
		createEReference(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__SIMPLE_TYPE);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__NAME);
		createEAttribute(selectedModelElementEClass, SELECTED_MODEL_ELEMENT__PSEUDO_CLASS);

		combinedModelElementEClass = createEClass(COMBINED_MODEL_ELEMENT);
		createEAttribute(combinedModelElementEClass, COMBINED_MODEL_ELEMENT__CHILD);
		createEAttribute(combinedModelElementEClass, COMBINED_MODEL_ELEMENT__NEXT);
		createEReference(combinedModelElementEClass, COMBINED_MODEL_ELEMENT__MODEL_ELEMENT);

		styleSpecificationPropertyEClass = createEClass(STYLE_SPECIFICATION_PROPERTY);

		stylePropertyFieldPartEClass = createEClass(STYLE_PROPERTY_FIELD_PART);
		createEReference(stylePropertyFieldPartEClass, STYLE_PROPERTY_FIELD_PART__PARTS);

		stylePropertyLeftEClass = createEClass(STYLE_PROPERTY_LEFT);
		createEReference(stylePropertyLeftEClass, STYLE_PROPERTY_LEFT__ATTACHMENT_SPECIFICATION);

		stylePropertyRightEClass = createEClass(STYLE_PROPERTY_RIGHT);
		createEReference(stylePropertyRightEClass, STYLE_PROPERTY_RIGHT__ATTACHMENT_SPECIFICATION);

		stylePropertyTopEClass = createEClass(STYLE_PROPERTY_TOP);
		createEReference(stylePropertyTopEClass, STYLE_PROPERTY_TOP__ATTACHMENT_SPECIFICATION);

		stylePropertyBottomEClass = createEClass(STYLE_PROPERTY_BOTTOM);
		createEReference(stylePropertyBottomEClass, STYLE_PROPERTY_BOTTOM__ATTACHMENT_SPECIFICATION);

		stylePropertyWidthEClass = createEClass(STYLE_PROPERTY_WIDTH);
		createEAttribute(stylePropertyWidthEClass, STYLE_PROPERTY_WIDTH__WIDTH);
		createEAttribute(stylePropertyWidthEClass, STYLE_PROPERTY_WIDTH__WIDTH_IN_CHAR);
		createEAttribute(stylePropertyWidthEClass, STYLE_PROPERTY_WIDTH__NEGATIVE_AFTER_CHAR);

		stylePropertyHeightEClass = createEClass(STYLE_PROPERTY_HEIGHT);
		createEAttribute(stylePropertyHeightEClass, STYLE_PROPERTY_HEIGHT__HEIGHT);

		stylePropertyTabulatorEClass = createEClass(STYLE_PROPERTY_TABULATOR);
		createEReference(stylePropertyTabulatorEClass, STYLE_PROPERTY_TABULATOR__TABULATORS);

		stylePropertyBorderEClass = createEClass(STYLE_PROPERTY_BORDER);
		createEAttribute(stylePropertyBorderEClass, STYLE_PROPERTY_BORDER__NONE_BORDER);
		createEAttribute(stylePropertyBorderEClass, STYLE_PROPERTY_BORDER__SOLID);

		stylePropertyForegroundColorEClass = createEClass(STYLE_PROPERTY_FOREGROUND_COLOR);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__RED);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__GREEN);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__BLUE);
		createEAttribute(stylePropertyForegroundColorEClass, STYLE_PROPERTY_FOREGROUND_COLOR__COLOR_ID);

		stylePropertyBackgroundColorEClass = createEClass(STYLE_PROPERTY_BACKGROUND_COLOR);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__RED);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__GREEN);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__BLUE);
		createEAttribute(stylePropertyBackgroundColorEClass, STYLE_PROPERTY_BACKGROUND_COLOR__COLOR_ID);

		stylePropertyWidgetVariantEClass = createEClass(STYLE_PROPERTY_WIDGET_VARIANT);
		createEAttribute(stylePropertyWidgetVariantEClass, STYLE_PROPERTY_WIDGET_VARIANT__NAME);

		stylePropertyImageEClass = createEClass(STYLE_PROPERTY_IMAGE);
		createEAttribute(stylePropertyImageEClass, STYLE_PROPERTY_IMAGE__IMAGE_URI);
		createEReference(stylePropertyImageEClass, STYLE_PROPERTY_IMAGE__IMAGE_URI_REFERENCE);

		stylePropertyAlignmentEClass = createEClass(STYLE_PROPERTY_ALIGNMENT);
		createEAttribute(stylePropertyAlignmentEClass, STYLE_PROPERTY_ALIGNMENT__ALIGNMENT);

		stylePropertyTableEClass = createEClass(STYLE_PROPERTY_TABLE);
		createEReference(stylePropertyTableEClass, STYLE_PROPERTY_TABLE__ONE_WAY);
		createEReference(stylePropertyTableEClass, STYLE_PROPERTY_TABLE__GRID_VISIBLE);

		stylePropertyFontEClass = createEClass(STYLE_PROPERTY_FONT);
		createEAttribute(stylePropertyFontEClass, STYLE_PROPERTY_FONT__STYLES);
		createEAttribute(stylePropertyFontEClass, STYLE_PROPERTY_FONT__SIZE);
		createEAttribute(stylePropertyFontEClass, STYLE_PROPERTY_FONT__NAME);

		stylePropertyCharacterWidthEClass = createEClass(STYLE_PROPERTY_CHARACTER_WIDTH);
		createEAttribute(stylePropertyCharacterWidthEClass, STYLE_PROPERTY_CHARACTER_WIDTH__CHAR_WIDTH);

		stylePropertyCharacterHeightEClass = createEClass(STYLE_PROPERTY_CHARACTER_HEIGHT);
		createEAttribute(stylePropertyCharacterHeightEClass, STYLE_PROPERTY_CHARACTER_HEIGHT__CHAR_HEIGHT);

		stylePropertyFlagEClass = createEClass(STYLE_PROPERTY_FLAG);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__ENABLED_EXPRESSION);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__MANDATORY_EXPRESSION);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__VISIBLE_EXPRESSION);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__COLLAPSED_EXPRESSION);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__AVAILABLE_EXPRESSION);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__EDITABLE_EXPRESSION);
		createEReference(stylePropertyFlagEClass, STYLE_PROPERTY_FLAG__TABABLE_EXPRESSION);

		stylePropertyLineWrapEClass = createEClass(STYLE_PROPERTY_LINE_WRAP);
		createEReference(stylePropertyLineWrapEClass, STYLE_PROPERTY_LINE_WRAP__WRAP);

		stylePropertyMultiLineEClass = createEClass(STYLE_PROPERTY_MULTI_LINE);
		createEReference(stylePropertyMultiLineEClass, STYLE_PROPERTY_MULTI_LINE__MULTI);

		stylePropertyBrowseControlEClass = createEClass(STYLE_PROPERTY_BROWSE_CONTROL);
		createEAttribute(stylePropertyBrowseControlEClass, STYLE_PROPERTY_BROWSE_CONTROL__CONTROL_TYPE);

		stylePropertyPagingEClass = createEClass(STYLE_PROPERTY_PAGING);
		createEAttribute(stylePropertyPagingEClass, STYLE_PROPERTY_PAGING__STYLE);
		createEAttribute(stylePropertyPagingEClass, STYLE_PROPERTY_PAGING__DEFAULT);

		stylePropertyPagingControlsEClass = createEClass(STYLE_PROPERTY_PAGING_CONTROLS);
		createEReference(stylePropertyPagingControlsEClass, STYLE_PROPERTY_PAGING_CONTROLS__CONTROLS);

		stylePropertyPagingPageSizeEClass = createEClass(STYLE_PROPERTY_PAGING_PAGE_SIZE);
		createEAttribute(stylePropertyPagingPageSizeEClass, STYLE_PROPERTY_PAGING_PAGE_SIZE__SIZE);
		createEAttribute(stylePropertyPagingPageSizeEClass, STYLE_PROPERTY_PAGING_PAGE_SIZE__MAX_SIZE);

		stylePropertyPagingJumpSizeEClass = createEClass(STYLE_PROPERTY_PAGING_JUMP_SIZE);
		createEAttribute(stylePropertyPagingJumpSizeEClass, STYLE_PROPERTY_PAGING_JUMP_SIZE__SIZE);

		stylePropertyPagingPositionEClass = createEClass(STYLE_PROPERTY_PAGING_POSITION);
		createEAttribute(stylePropertyPagingPositionEClass, STYLE_PROPERTY_PAGING_POSITION__TOP);
		createEAttribute(stylePropertyPagingPositionEClass, STYLE_PROPERTY_PAGING_POSITION__BOTTOM);

		stylePropertyPagingCustomizerImageEClass = createEClass(STYLE_PROPERTY_PAGING_CUSTOMIZER_IMAGE);
		createEAttribute(stylePropertyPagingCustomizerImageEClass, STYLE_PROPERTY_PAGING_CUSTOMIZER_IMAGE__IMAGE_URI);
		createEReference(stylePropertyPagingCustomizerImageEClass, STYLE_PROPERTY_PAGING_CUSTOMIZER_IMAGE__IMAGE_URI_REFERENCE);

		pagingControlEClass = createEClass(PAGING_CONTROL);
		createEAttribute(pagingControlEClass, PAGING_CONTROL__CONTROL);
		createEAttribute(pagingControlEClass, PAGING_CONTROL__END);
		createEAttribute(pagingControlEClass, PAGING_CONTROL__NEXT);
		createEAttribute(pagingControlEClass, PAGING_CONTROL__CUSTOMIZE);
		createEAttribute(pagingControlEClass, PAGING_CONTROL__EXPORT);

		stylePropertyTableCustomizerEClass = createEClass(STYLE_PROPERTY_TABLE_CUSTOMIZER);
		createEReference(stylePropertyTableCustomizerEClass, STYLE_PROPERTY_TABLE_CUSTOMIZER__TABLE_CUSTOMIZER);

		stylePropertyScrollableEClass = createEClass(STYLE_PROPERTY_SCROLLABLE);
		createEAttribute(stylePropertyScrollableEClass, STYLE_PROPERTY_SCROLLABLE__NOT_SCROLLABLE);
		createEAttribute(stylePropertyScrollableEClass, STYLE_PROPERTY_SCROLLABLE__ORIENTATIONS);

		stylePropertyEnumerationDisplayTypeEClass = createEClass(STYLE_PROPERTY_ENUMERATION_DISPLAY_TYPE);
		createEAttribute(stylePropertyEnumerationDisplayTypeEClass, STYLE_PROPERTY_ENUMERATION_DISPLAY_TYPE__TYPE);

		stylePropertyEnumerationSortOrderEClass = createEClass(STYLE_PROPERTY_ENUMERATION_SORT_ORDER);
		createEAttribute(stylePropertyEnumerationSortOrderEClass, STYLE_PROPERTY_ENUMERATION_SORT_ORDER__SORTORDER);

		stylePropertyOtherEClass = createEClass(STYLE_PROPERTY_OTHER);
		createEReference(stylePropertyOtherEClass, STYLE_PROPERTY_OTHER__DYNAMIC_TAB_PAGE);
		createEReference(stylePropertyOtherEClass, STYLE_PROPERTY_OTHER__THOUSAND_SEPERATOR);
		createEReference(stylePropertyOtherEClass, STYLE_PROPERTY_OTHER__SPACING);
		createEReference(stylePropertyOtherEClass, STYLE_PROPERTY_OTHER__MARGIN);
		createEAttribute(stylePropertyOtherEClass, STYLE_PROPERTY_OTHER__WIDGET_EFFECTS);

		tableCustomizerEClass = createEClass(TABLE_CUSTOMIZER);
		createEAttribute(tableCustomizerEClass, TABLE_CUSTOMIZER__INSTANCE_TYPE);

		commandImplementationEClass = createEClass(COMMAND_IMPLEMENTATION);
		createEReference(commandImplementationEClass, COMMAND_IMPLEMENTATION__STATEMENTS);

		statementEClass = createEClass(STATEMENT);

		statementVariableEClass = createEClass(STATEMENT_VARIABLE);
		createEReference(statementVariableEClass, STATEMENT_VARIABLE__REFERENCE);

		iReferenceableByStatementVariableEClass = createEClass(IREFERENCEABLE_BY_STATEMENT_VARIABLE);

		assignStatementEClass = createEClass(ASSIGN_STATEMENT);
		createEReference(assignStatementEClass, ASSIGN_STATEMENT__VARIABLE_ASSIGNMENTS);

		variableAssignmentEClass = createEClass(VARIABLE_ASSIGNMENT);
		createEReference(variableAssignmentEClass, VARIABLE_ASSIGNMENT__LEFT);
		createEReference(variableAssignmentEClass, VARIABLE_ASSIGNMENT__VALUE);
		createEReference(variableAssignmentEClass, VARIABLE_ASSIGNMENT__SERVICE_CALL);

		variableValueEClass = createEClass(VARIABLE_VALUE);
		createEReference(variableValueEClass, VARIABLE_VALUE__VARIABLE);
		createEAttribute(variableValueEClass, VARIABLE_VALUE__ACCESS);
		createEAttribute(variableValueEClass, VARIABLE_VALUE__STRING_LITERAL);
		createEAttribute(variableValueEClass, VARIABLE_VALUE__INTEGER_LITERAL);

		executeStatementEClass = createEClass(EXECUTE_STATEMENT);
		createEReference(executeStatementEClass, EXECUTE_STATEMENT__SERVICE);
		createEReference(executeStatementEClass, EXECUTE_STATEMENT__OPERATION);
		createEReference(executeStatementEClass, EXECUTE_STATEMENT__DELEGATE_OPERATION);
		createEReference(executeStatementEClass, EXECUTE_STATEMENT__PARAMETER_VALUES);

		invokeStatementEClass = createEClass(INVOKE_STATEMENT);
		createEReference(invokeStatementEClass, INVOKE_STATEMENT__INVOKE_ABLE);
		createEReference(invokeStatementEClass, INVOKE_STATEMENT__TARGET_PANEL);
		createEReference(invokeStatementEClass, INVOKE_STATEMENT__PROPERTY_JOIN);

		callStatementEClass = createEClass(CALL_STATEMENT);
		createEReference(callStatementEClass, CALL_STATEMENT__COMMAND);
		createEReference(callStatementEClass, CALL_STATEMENT__LEFT);
		createEAttribute(callStatementEClass, CALL_STATEMENT__JAVA_COMMAND);
		createEReference(callStatementEClass, CALL_STATEMENT__PARAMETER_VALUES);

		invokeableEClass = createEClass(INVOKEABLE);

		proxyEClass = createEClass(PROXY);
		createEAttribute(proxyEClass, PROXY__URL);
		createEReference(proxyEClass, PROXY__PROPERTIES);

		propertyJoinEClass = createEClass(PROPERTY_JOIN);
		createEReference(propertyJoinEClass, PROPERTY_JOIN__VARIABLE);
		createEAttribute(propertyJoinEClass, PROPERTY_JOIN__ACCESS);
		createEAttribute(propertyJoinEClass, PROPERTY_JOIN__STRING_LITERAL);
		createEAttribute(propertyJoinEClass, PROPERTY_JOIN__INTEGER_LITERAL);
		createEAttribute(propertyJoinEClass, PROPERTY_JOIN__JOIN_DIRECTION);
		createEReference(propertyJoinEClass, PROPERTY_JOIN__PROPERTY);

		// Create enums
		eventTypeEEnum = createEEnum(EVENT_TYPE);
		predefinedCommandEEnum = createEEnum(PREDEFINED_COMMAND);
		attachmentOwnPositionEEnum = createEEnum(ATTACHMENT_OWN_POSITION);
		relativeSiblingEEnum = createEEnum(RELATIVE_SIBLING);
		attachmentSiblingPositionEEnum = createEEnum(ATTACHMENT_SIBLING_POSITION);
		contentAlignmentEEnum = createEEnum(CONTENT_ALIGNMENT);
		tabAlignmentEEnum = createEEnum(TAB_ALIGNMENT);
		horizontalAlignmentEEnum = createEEnum(HORIZONTAL_ALIGNMENT);
		verticalAlignmentEEnum = createEEnum(VERTICAL_ALIGNMENT);
		orientationEEnum = createEEnum(ORIENTATION);
		controlTypeEEnum = createEEnum(CONTROL_TYPE);
		fieldPartEEnum = createEEnum(FIELD_PART);
		titleButtonEEnum = createEEnum(TITLE_BUTTON);
		joinDirectionEEnum = createEEnum(JOIN_DIRECTION);
		styleSelectorTypeEEnum = createEEnum(STYLE_SELECTOR_TYPE);
		browseControlTypeEEnum = createEEnum(BROWSE_CONTROL_TYPE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	private boolean isInitialized = false;

	/**
	 * Complete the initialization of the package and its meta-model.  This
	 * method is guarded to have no affect on any invocation but its first.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void initializePackageContents() {
		if (isInitialized) return;
		isInitialized = true;

		// Initialize package
		setName(eNAME);
		setNsPrefix(eNS_PREFIX);
		setNsURI(eNS_URI);

		// Obtain other dependent packages
		CorePackage theCorePackage = (CorePackage)EPackage.Registry.INSTANCE.getEPackage(CorePackage.eNS_URI);
		GuidesignPackage theGuidesignPackage = (GuidesignPackage)EPackage.Registry.INSTANCE.getEPackage(GuidesignPackage.eNS_URI);
		DomPackage theDomPackage = (DomPackage)EPackage.Registry.INSTANCE.getEPackage(DomPackage.eNS_URI);
		FlexPackage theFlexPackage = (FlexPackage)EPackage.Registry.INSTANCE.getEPackage(FlexPackage.eNS_URI);

		// Create type parameters

		// Set bounds for type parameters

		// Add supertypes to classes
		enabledFlagEClass.getESuperTypes().add(this.getFieldFlag());
		mandatoryFlagEClass.getESuperTypes().add(this.getFieldFlag());
		visibleFlagEClass.getESuperTypes().add(this.getFieldFlag());
		collapsedFlagEClass.getESuperTypes().add(this.getFieldFlag());
		editableFlagEClass.getESuperTypes().add(this.getFieldFlag());
		objectPropertyEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		objectPropertyEClass.getESuperTypes().add(this.getDataBindingElement());
		objectPropertyEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		objectPropertyEClass.getESuperTypes().add(theCorePackage.getIDocumentable());
		componentEClass.getESuperTypes().add(theCorePackage.getModelElement());
		componentEClass.getESuperTypes().add(this.getReferencedXMAGuiElement());
		componentEClass.getESuperTypes().add(this.getInvokeable());
		componentEClass.getESuperTypes().add(theCorePackage.getIDocumentable());
		referencedXMAPageEClass.getESuperTypes().add(this.getReferencedXMAGuiElement());
		referencedXMAPageEClass.getESuperTypes().add(this.getPage());
		referencedXMACompositeEClass.getESuperTypes().add(this.getReferencedXMAGuiElement());
		referencedXMACompositeEClass.getESuperTypes().add(this.getIComposite());
		dataObjectVariableEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		dataObjectVariableEClass.getESuperTypes().add(theCorePackage.getIDocumentable());
		labelPropertyEClass.getESuperTypes().add(this.getTextProperty());
		tooltipPropertyEClass.getESuperTypes().add(this.getTextProperty());
		unitPropertyEClass.getESuperTypes().add(this.getTextProperty());
		fieldReferenceEClass.getESuperTypes().add(this.getIField());
		customizeableFieldEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		customizeableFieldEClass.getESuperTypes().add(this.getIField());
		customizeableFieldEClass.getESuperTypes().add(this.getSimpleElement());
		customizeableFieldEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		commandEClass.getESuperTypes().add(theCorePackage.getIDocumentable());
		controlEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		xmaWidgetEventMappingEClass.getESuperTypes().add(this.getControlEventMapping());
		guiElementEventMappingEClass.getESuperTypes().add(this.getControlEventMapping());
		initEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		enterEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		leaveEventMappingEClass.getESuperTypes().add(this.getEventMapping());
		pageEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		pageEClass.getESuperTypes().add(this.getPageDefinition());
		pageEClass.getESuperTypes().add(this.getComposite());
		pageEClass.getESuperTypes().add(this.getInvokeable());
		pageEClass.getESuperTypes().add(theCorePackage.getIDocumentable());
		xmadslPageEClass.getESuperTypes().add(this.getPage());
		tabMenuEClass.getESuperTypes().add(this.getMenu());
		dropDownMenuEClass.getESuperTypes().add(this.getMenu());
		treeMenuEClass.getESuperTypes().add(this.getMenu());
		menuItemEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		pageCustomizationEClass.getESuperTypes().add(this.getPageDefinition());
		compositeEClass.getESuperTypes().add(this.getGuiElement());
		composedElementEClass.getESuperTypes().add(this.getGuiElement());
		composedElementEClass.getESuperTypes().add(this.getIElementOnWhichCanBeAttached());
		complexElementEClass.getESuperTypes().add(this.getComposedElement());
		simpleElementEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		simpleElementEClass.getESuperTypes().add(this.getComposedElement());
		guiElementWithEvent_dummyEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		textEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		textEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		textEClass.getESuperTypes().add(this.getDataBindingElement());
		textEClass.getESuperTypes().add(this.getSimpleElement());
		textEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		comboEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		comboEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		comboEClass.getESuperTypes().add(this.getDataBindingElement());
		comboEClass.getESuperTypes().add(this.getSimpleElement());
		comboEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		checkBoxEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		checkBoxEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		checkBoxEClass.getESuperTypes().add(this.getDataBindingElement());
		checkBoxEClass.getESuperTypes().add(this.getSimpleElement());
		checkBoxEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		listBoxEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		listBoxEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		listBoxEClass.getESuperTypes().add(this.getSimpleElement());
		listBoxEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		includePanelEClass.getESuperTypes().add(this.getSimpleElement());
		treeEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		treeEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		treeEClass.getESuperTypes().add(this.getSimpleElement());
		treeEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		radioButtonEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		radioButtonEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		radioButtonEClass.getESuperTypes().add(this.getDataBindingElement());
		radioButtonEClass.getESuperTypes().add(this.getSimpleElement());
		radioButtonEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		seperatorEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		seperatorEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		seperatorEClass.getESuperTypes().add(this.getSimpleElement());
		labelEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		labelEClass.getESuperTypes().add(this.getSimpleElement());
		labelEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		imageEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		imageEClass.getESuperTypes().add(this.getSimpleElement());
		buttonEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		buttonEClass.getESuperTypes().add(this.getSimpleElement());
		iCompositeEClass.getESuperTypes().add(this.getComposite());
		iCompositeEClass.getESuperTypes().add(this.getComplexElement());
		xmadslCompositeEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		xmadslCompositeEClass.getESuperTypes().add(this.getIComposite());
		splitPanelEClass.getESuperTypes().add(this.getXmadslComposite());
		referencedCompositeEClass.getESuperTypes().add(this.getIComposite());
		tableComboEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		tableComboEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		tableComboEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		tableComboEClass.getESuperTypes().add(this.getComplexElement());
		tableComboEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		tableEClass.getESuperTypes().add(this.getIGuiElementWithEvent());
		tableEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		tableEClass.getESuperTypes().add(theCorePackage.getReferenceableByXmadslVariable());
		tableEClass.getESuperTypes().add(this.getComplexElement());
		tableEClass.getESuperTypes().add(this.getIReferenceableByStatementVariable());
		iTabFolderEClass.getESuperTypes().add(this.getComplexElement());
		tabFolderEClass.getESuperTypes().add(this.getIElementWithLayoutData());
		tabFolderEClass.getESuperTypes().add(this.getITabFolder());
		referencedTabFolderEClass.getESuperTypes().add(this.getITabFolder());
		iTabPageEClass.getESuperTypes().add(this.getPage());
		tabPageEClass.getESuperTypes().add(this.getITabPage());
		referencedTabPageEClass.getESuperTypes().add(this.getITabPage());
		heightPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		heightPropertyEClass.getESuperTypes().add(this.getPageProperty());
		widthPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		widthPropertyEClass.getESuperTypes().add(this.getPageProperty());
		stylePropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		stylePropertyEClass.getESuperTypes().add(this.getPageProperty());
		modalityPropertyEClass.getESuperTypes().add(this.getPageProperty());
		imagePropertyEClass.getESuperTypes().add(this.getPageProperty());
		titleButtonsPropertyEClass.getESuperTypes().add(this.getPageProperty());
		resizeablePropertyEClass.getESuperTypes().add(this.getPageProperty());
		statusBarPropertyEClass.getESuperTypes().add(this.getPageProperty());
		centerPropertyEClass.getESuperTypes().add(this.getPageProperty());
		alignmentPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		iAttachmentPositionEClass.getESuperTypes().add(this.getIElementOnWhichCanBeAttached());
		tabulatorPositionEClass.getESuperTypes().add(this.getIAttachmentPosition());
		attachmentPositionEClass.getESuperTypes().add(this.getIAttachmentPosition());
		attachmentPropertyEClass.getESuperTypes().add(this.getLayoutDataProperty());
		noneAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		parentAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		siblingAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		absoluteSiblingAttachmentEClass.getESuperTypes().add(this.getSiblingAttachment());
		relativeSiblingAttachmentEClass.getESuperTypes().add(this.getSiblingAttachment());
		tabulatorAttachmentEClass.getESuperTypes().add(this.getAttachmentSpecification());
		fieldVariableEClass.getESuperTypes().add(theCorePackage.getVariable());
		xmaVariableEClass.getESuperTypes().add(theCorePackage.getVariable());
		customizationOfCompositeEClass.getESuperTypes().add(this.getCustomizationOfGuiElement());
		customizationOfTabFolderEClass.getESuperTypes().add(this.getCustomizationOfGuiElement());
		styleSpecificationEClass.getESuperTypes().add(theCorePackage.getStyle());
		stylePropertyFieldPartEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyLeftEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyRightEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyTopEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBottomEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyWidthEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyHeightEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyTabulatorEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBorderEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyForegroundColorEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBackgroundColorEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyWidgetVariantEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyImageEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyAlignmentEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyTableEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyFontEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyCharacterWidthEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyCharacterHeightEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyFlagEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyLineWrapEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyMultiLineEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyBrowseControlEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyPagingEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyPagingControlsEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyPagingPageSizeEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyPagingJumpSizeEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyPagingPositionEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyPagingCustomizerImageEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyTableCustomizerEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyScrollableEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyEnumerationDisplayTypeEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyEnumerationSortOrderEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		stylePropertyOtherEClass.getESuperTypes().add(this.getStyleSpecificationProperty());
		tableCustomizerEClass.getESuperTypes().add(theCorePackage.getModelElement());
		assignStatementEClass.getESuperTypes().add(this.getStatement());
		executeStatementEClass.getESuperTypes().add(this.getStatement());
		invokeStatementEClass.getESuperTypes().add(this.getStatement());
		callStatementEClass.getESuperTypes().add(this.getStatement());
		proxyEClass.getESuperTypes().add(theCorePackage.getModelElement());
		proxyEClass.getESuperTypes().add(this.getInvokeable());

		// Initialize classes and features; add operations and parameters
		initEClass(iGuiElementWithEventEClass, IGuiElementWithEvent.class, "IGuiElementWithEvent", IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(iElementWithLayoutDataEClass, IElementWithLayoutData.class, "IElementWithLayoutData", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getIElementWithLayoutData_LayoutProperties(), this.getLayoutDataProperty(), null, "layoutProperties", null, 0, -1, IElementWithLayoutData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIElementWithLayoutData_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, IElementWithLayoutData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIElementWithLayoutData_TabableFlag(), this.getTabableFlag(), null, "tabableFlag", null, 0, 1, IElementWithLayoutData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(dataBindingElementEClass, DataBindingElement.class, "DataBindingElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(fieldFlagEClass, FieldFlag.class, "FieldFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getFieldFlag_Expression(), theCorePackage.getEqualityExpr(), null, "expression", null, 0, 1, FieldFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(enabledFlagEClass, EnabledFlag.class, "EnabledFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(mandatoryFlagEClass, MandatoryFlag.class, "MandatoryFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(visibleFlagEClass, VisibleFlag.class, "VisibleFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(collapsedFlagEClass, CollapsedFlag.class, "CollapsedFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCollapsedFlag_AvailableSpecified(), ecorePackage.getEBoolean(), "availableSpecified", null, 0, 1, CollapsedFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(editableFlagEClass, EditableFlag.class, "EditableFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(tabableFlagEClass, TabableFlag.class, "TabableFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTabableFlag_Value(), theCorePackage.getBoolLiteral(), null, "value", null, 0, 1, TabableFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(multiselectionFlagEClass, MultiselectionFlag.class, "MultiselectionFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getMultiselectionFlag_MultiSelection(), theCorePackage.getBoolLiteral(), null, "multiSelection", null, 0, 1, MultiselectionFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(strictFlagEClass, StrictFlag.class, "StrictFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStrictFlag_Strict(), theCorePackage.getBoolLiteral(), null, "strict", null, 0, 1, StrictFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(labelTextEClass, LabelText.class, "LabelText", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getLabelText_Label(), ecorePackage.getEString(), "label", null, 0, 1, LabelText.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLabelText_ReferencedLabelString(), this.getStringDefinition(), null, "referencedLabelString", null, 0, 1, LabelText.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(objectPropertyEClass, ObjectProperty.class, "ObjectProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getObjectProperty_Public(), ecorePackage.getEBoolean(), "public", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getObjectProperty_Type(), theCorePackage.getSimpleType(), null, "type", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getObjectProperty_Name(), ecorePackage.getEString(), "name", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getObjectProperty_DefaultValue(), ecorePackage.getEString(), "defaultValue", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getObjectProperty_ReadOnly(), theCorePackage.getBoolLiteral(), null, "readOnly", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getObjectProperty_Required(), theCorePackage.getBoolLiteral(), null, "required", null, 0, 1, ObjectProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(componentEClass, Component.class, "Component", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getComponent_XmaComponent(), theGuidesignPackage.getXMAComponent(), null, "xmaComponent", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_StyleProperty(), this.getStyleProperty(), null, "styleProperty", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_DefaultPage(), this.getXmadslPage(), null, "defaultPage", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Dependencies(), theDomPackage.getDependant(), null, "dependencies", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_ComponentProperties(), this.getObjectProperty(), null, "componentProperties", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Dataobjects(), this.getDataObjectVariable(), null, "dataobjects", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Commands(), this.getCommand(), null, "commands", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Events(), this.getEventMappingList(), null, "events", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_ConditionsBlock(), theCorePackage.getConditionsBlock(), null, "conditionsBlock", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Pages(), this.getPage(), null, "pages", null, 0, -1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComponent_Definitions(), this.getDefinitions(), null, "definitions", null, 0, 1, Component.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedXMAGuiElementEClass, ReferencedXMAGuiElement.class, "ReferencedXMAGuiElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(referencedXMAPageEClass, ReferencedXMAPage.class, "ReferencedXMAPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedXMAPage_XmaPage(), theGuidesignPackage.getXMAPage(), null, "xmaPage", null, 0, 1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_Dependencies(), theDomPackage.getDependant(), null, "dependencies", null, 0, -1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_LogicBlock(), this.getLogicBlock(), null, "logicBlock", null, 0, 1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAPage_GrayLogic(), this.getGrayLogic(), null, "grayLogic", null, 0, 1, ReferencedXMAPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedXMACompositeEClass, ReferencedXMAComposite.class, "ReferencedXMAComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedXMAComposite_XmaComposite(), theGuidesignPackage.getXMAComposite(), null, "xmaComposite", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAComposite_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getReferencedXMAComposite_TabableFlag(), this.getTabableFlag(), null, "tabableFlag", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getReferencedXMAComposite_ReferencingPageComposite(), ecorePackage.getEBoolean(), "referencingPageComposite", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getReferencedXMAComposite_HorizontalPartitioning(), ecorePackage.getEBoolean(), "horizontalPartitioning", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getReferencedXMAComposite_VerticalPartitioning(), ecorePackage.getEBoolean(), "verticalPartitioning", null, 0, 1, ReferencedXMAComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(grayLogicEClass, GrayLogic.class, "GrayLogic", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getGrayLogic_WidgetGrayLogic(), this.getXMAWidgetGrayLogic(), null, "widgetGrayLogic", null, 0, -1, GrayLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmaWidgetGrayLogicEClass, XMAWidgetGrayLogic.class, "XMAWidgetGrayLogic", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXMAWidgetGrayLogic_XmaWidget(), theGuidesignPackage.getXMAWidget(), null, "xmaWidget", null, 0, 1, XMAWidgetGrayLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXMAWidgetGrayLogic_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, XMAWidgetGrayLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(dataObjectVariableEClass, DataObjectVariable.class, "DataObjectVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDataObjectVariable_Type(), theDomPackage.getComplexType(), null, "type", null, 0, 1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getDataObjectVariable_IsCollection(), ecorePackage.getEBoolean(), "isCollection", null, 0, 1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getDataObjectVariable_Name(), ecorePackage.getEString(), "name", null, 0, 1, DataObjectVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(textPropertyEClass, TextProperty.class, "TextProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(labelPropertyEClass, LabelProperty.class, "LabelProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getLabelProperty_LabelText(), ecorePackage.getEString(), "labelText", null, 0, 1, LabelProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLabelProperty_ReferencedLabelString(), this.getStringDefinition(), null, "referencedLabelString", null, 0, 1, LabelProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tooltipPropertyEClass, TooltipProperty.class, "TooltipProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTooltipProperty_TooltipText(), ecorePackage.getEString(), "tooltipText", null, 0, 1, TooltipProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTooltipProperty_ReferencedTooltipString(), this.getStringDefinition(), null, "referencedTooltipString", null, 0, 1, TooltipProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(unitPropertyEClass, UnitProperty.class, "UnitProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getUnitProperty_UnitLabelText(), ecorePackage.getEString(), "unitLabelText", null, 0, 1, UnitProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getUnitProperty_ReferencedUnitLabelString(), this.getStringDefinition(), null, "referencedUnitLabelString", null, 0, 1, UnitProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iFieldEClass, IField.class, "IField", IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getIField_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, IField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIField_Feature(), this.getFieldFeature(), null, "feature", null, 0, 1, IField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldFeatureEClass, FieldFeature.class, "FieldFeature", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getFieldFeature_Feature(), theDomPackage.getPresentableFeature(), null, "feature", null, 0, 1, FieldFeature.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getFieldFeature_SubFeature(), this.getFieldFeature(), null, "subFeature", null, 0, 1, FieldFeature.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldReferenceEClass, FieldReference.class, "FieldReference", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(customizeableFieldEClass, CustomizeableField.class, "CustomizeableField", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCustomizeableField_ControlType(), this.getControlType(), "controlType", null, 0, 1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_Parts(), this.getFieldPartSpecification(), null, "parts", null, 0, -1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_MultiLine(), theCorePackage.getBoolLiteral(), null, "multiLine", null, 0, 1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_Constraints(), theCorePackage.getConstraint(), null, "constraints", null, 0, -1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeableField_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, CustomizeableField.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldPartSpecificationEClass, FieldPartSpecification.class, "FieldPartSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getFieldPartSpecification_Part(), this.getFieldPart(), "part", null, 0, 1, FieldPartSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getFieldPartSpecification_LayoutProperties(), this.getLayoutDataProperty(), null, "layoutProperties", null, 0, -1, FieldPartSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(commandEClass, Command.class, "Command", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCommand_Doc(), ecorePackage.getEString(), "doc", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCommand_ClientOnly(), ecorePackage.getEBoolean(), "clientOnly", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCommand_Name(), ecorePackage.getEString(), "name", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCommand_Implementation(), this.getCommandImplementation(), null, "implementation", null, 0, 1, Command.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(eventFunctionEClass, EventFunction.class, "EventFunction", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getEventFunction_Command(), this.getCommand(), null, "command", null, 0, 1, EventFunction.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getEventFunction_PredefinedCommand(), this.getPredefinedCommand(), "predefinedCommand", null, 0, 1, EventFunction.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(eventMappingListEClass, EventMappingList.class, "EventMappingList", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getEventMappingList_Mapping(), this.getEventMapping(), null, "mapping", null, 0, -1, EventMappingList.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(eventMappingEClass, EventMapping.class, "EventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getEventMapping_EventFunctions(), this.getEventFunction(), null, "eventFunctions", null, 0, -1, EventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(controlEventMappingEClass, ControlEventMapping.class, "ControlEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getControlEventMapping_Event(), this.getEventType(), "event", null, 0, 1, ControlEventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmaWidgetEventMappingEClass, XMAWidgetEventMapping.class, "XMAWidgetEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXMAWidgetEventMapping_Control(), theGuidesignPackage.getXMAWidget(), null, "control", null, 0, 1, XMAWidgetEventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(guiElementEventMappingEClass, GuiElementEventMapping.class, "GuiElementEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getGuiElementEventMapping_Control(), this.getIGuiElementWithEvent(), null, "control", null, 0, 1, GuiElementEventMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(initEventMappingEClass, InitEventMapping.class, "InitEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(enterEventMappingEClass, EnterEventMapping.class, "EnterEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(leaveEventMappingEClass, LeaveEventMapping.class, "LeaveEventMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(dataMappingListEClass, DataMappingList.class, "DataMappingList", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDataMappingList_Mapping(), this.getDataMapping(), null, "mapping", null, 0, -1, DataMappingList.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(dataMappingEClass, DataMapping.class, "DataMapping", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDataMapping_Control(), theGuidesignPackage.getIBDAttachable(), null, "control", null, 0, 1, DataMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDataMapping_BindingElement(), this.getDataBindingElement(), null, "bindingElement", null, 0, 1, DataMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDataMapping_Field(), this.getFieldReference(), null, "field", null, 0, 1, DataMapping.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(pageDefinitionEClass, PageDefinition.class, "PageDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(pageEClass, Page.class, "Page", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getPage_ComposeLayout(), this.getComposeData(), null, "composeLayout", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Dataobjects(), this.getDataObjectVariable(), null, "dataobjects", null, 0, -1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_DataMapping(), this.getDataMappingList(), null, "dataMapping", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Commands(), this.getCommand(), null, "commands", null, 0, -1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Events(), this.getEventMappingList(), null, "events", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_ConditionsBlock(), theCorePackage.getConditionsBlock(), null, "conditionsBlock", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPage_Definitions(), this.getDefinitions(), null, "definitions", null, 0, 1, Page.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmadslPageEClass, XmadslPage.class, "XmadslPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getXmadslPage_Abstract(), ecorePackage.getEBoolean(), "abstract", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmadslPage_Name(), ecorePackage.getEString(), "name", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_PageProperty(), this.getPageProperty(), null, "pageProperty", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Template(), this.getPage(), null, "template", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Dependencies(), theDomPackage.getDependant(), null, "dependencies", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_PageProperties(), this.getObjectProperty(), null, "pageProperties", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Menus(), this.getMenu(), null, "menus", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_LogicBlock(), this.getLogicBlock(), null, "logicBlock", null, 0, 1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslPage_Extensions(), this.getCustomizationOfGuiElement(), null, "extensions", null, 0, -1, XmadslPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(menuEClass, Menu.class, "Menu", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getMenu_MenuItems(), this.getMenuItem(), null, "menuItems", null, 0, -1, Menu.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabMenuEClass, TabMenu.class, "TabMenu", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTabMenu_Panel(), this.getXmadslComposite(), null, "panel", null, 0, 1, TabMenu.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTabMenu_Alignment(), this.getTabAlignment(), "alignment", null, 0, 1, TabMenu.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabMenu_Defaultselection(), this.getMenuItem(), null, "defaultselection", null, 0, 1, TabMenu.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(dropDownMenuEClass, DropDownMenu.class, "DropDownMenu", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(treeMenuEClass, TreeMenu.class, "TreeMenu", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTreeMenu_Panel(), this.getXmadslComposite(), null, "panel", null, 0, 1, TreeMenu.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(menuItemEClass, MenuItem.class, "MenuItem", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getMenuItem_Name(), ecorePackage.getEString(), "name", null, 0, 1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getMenuItem_Label(), ecorePackage.getEString(), "label", null, 0, 1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getMenuItem_Accelerator(), ecorePackage.getEString(), "accelerator", null, 0, 1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getMenuItem_Image(), ecorePackage.getEString(), "image", null, 0, 1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getMenuItem_Defaultselection(), this.getMenuItem(), null, "defaultselection", null, 0, 1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getMenuItem_Enabled(), this.getEnabledFlag(), null, "enabled", null, 0, 1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getMenuItem_MenuItem(), this.getMenuItem(), null, "menuItem", null, 0, -1, MenuItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(logicBlockEClass, LogicBlock.class, "LogicBlock", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getLogicBlock_LogicItem(), this.getLogicItem(), null, "logicItem", null, 0, -1, LogicBlock.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLogicBlock_ConditionedLogic(), this.getConditionedLogic(), null, "conditionedLogic", null, 0, -1, LogicBlock.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(conditionedLogicEClass, ConditionedLogic.class, "ConditionedLogic", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getConditionedLogic_StatusFlag(), theCorePackage.getStatusFlag(), null, "statusFlag", null, 0, 1, ConditionedLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getConditionedLogic_Default(), ecorePackage.getEBoolean(), "default", null, 0, 1, ConditionedLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getConditionedLogic_LogicItem(), this.getLogicItem(), null, "logicItem", null, 0, -1, ConditionedLogic.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(logicItemEClass, LogicItem.class, "LogicItem", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getLogicItem_GuiElement(), this.getGuiElement(), null, "guiElement", null, 0, 1, LogicItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLogicItem_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, LogicItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLogicItem_TabableFlag(), this.getTabableFlag(), null, "tabableFlag", null, 0, 1, LogicItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLogicItem_ColumnOrder(), this.getColumnOrder(), null, "columnOrder", null, 0, 1, LogicItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLogicItem_TabSettings(), this.getTabSetting(), null, "tabSettings", null, 0, -1, LogicItem.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(columnOrderEClass, ColumnOrder.class, "ColumnOrder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getColumnOrder_ColumnInfos(), this.getColumnInfo(), null, "columnInfos", null, 0, -1, ColumnOrder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabSettingEClass, TabSetting.class, "TabSetting", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTabSetting_OrderedElements(), this.getGuiElement(), null, "orderedElements", null, 0, -1, TabSetting.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(columnInfoEClass, ColumnInfo.class, "ColumnInfo", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getColumnInfo_Columns(), this.getTableColumn(), null, "columns", null, 0, 1, ColumnInfo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getColumnInfo_Width(), ecorePackage.getEInt(), "width", null, 0, 1, ColumnInfo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getColumnInfo_RelativeWidth(), ecorePackage.getEBoolean(), "relativeWidth", null, 0, 1, ColumnInfo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getColumnInfo_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, ColumnInfo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(pageCustomizationEClass, PageCustomization.class, "PageCustomization", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getPageCustomization_PageToCustomize(), this.getPage(), null, "pageToCustomize", null, 0, 1, PageCustomization.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPageCustomization_Composites(), this.getComposite(), null, "composites", null, 0, -1, PageCustomization.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(guiElementEClass, GuiElement.class, "GuiElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(compositeEClass, Composite.class, "Composite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getComposite_Content(), this.getContent(), null, "content", null, 0, 1, Composite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(composedElementEClass, ComposedElement.class, "ComposedElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(complexElementEClass, ComplexElement.class, "ComplexElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(simpleElementEClass, SimpleElement.class, "SimpleElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(guiElementWithEvent_dummyEClass, GuiElementWithEvent_dummy.class, "GuiElementWithEvent_dummy", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getGuiElementWithEvent_dummy_DummyProperty(), ecorePackage.getEString(), "dummyProperty", null, 0, 1, GuiElementWithEvent_dummy.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(textEClass, Text.class, "Text", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getText_Name(), ecorePackage.getEString(), "name", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getText_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_MultiLine(), theCorePackage.getBoolLiteral(), null, "multiLine", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_Constraints(), theCorePackage.getConstraint(), null, "constraints", null, 0, -1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_Type(), theCorePackage.getDataTypeAndTypeParameter(), null, "type", null, 0, 1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getText_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, Text.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(comboEClass, Combo.class, "Combo", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCombo_Name(), ecorePackage.getEString(), "name", null, 0, 1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_Constraints(), theCorePackage.getConstraint(), null, "constraints", null, 0, -1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_Type(), theCorePackage.getDataTypeAndTypeParameter(), null, "type", null, 0, 1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_StrictFlag(), this.getStrictFlag(), null, "strictFlag", null, 0, 1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombo_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, Combo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(checkBoxEClass, CheckBox.class, "CheckBox", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCheckBox_Name(), ecorePackage.getEString(), "name", null, 0, 1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_Constraints(), theCorePackage.getConstraint(), null, "constraints", null, 0, -1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCheckBox_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, CheckBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(listBoxEClass, ListBox.class, "ListBox", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getListBox_Name(), ecorePackage.getEString(), "name", null, 0, 1, ListBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getListBox_MultiselectionFlag(), this.getMultiselectionFlag(), null, "multiselectionFlag", null, 0, 1, ListBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getListBox_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, ListBox.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(includePanelEClass, IncludePanel.class, "IncludePanel", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getIncludePanel_Name(), ecorePackage.getEString(), "name", null, 0, 1, IncludePanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIncludePanel_XmaComponent(), theGuidesignPackage.getXMAComponent(), null, "xmaComponent", null, 0, 1, IncludePanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIncludePanel_Component(), this.getComponent(), null, "component", null, 0, 1, IncludePanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIncludePanel_EmbeddedPage(), theGuidesignPackage.getEmbeddedPage(), null, "embeddedPage", null, 0, 1, IncludePanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIncludePanel_XmadslPage(), this.getXmadslPage(), null, "xmadslPage", null, 0, 1, IncludePanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(treeEClass, Tree.class, "Tree", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTree_Name(), ecorePackage.getEString(), "name", null, 0, 1, Tree.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTree_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, Tree.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(radioButtonEClass, RadioButton.class, "RadioButton", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getRadioButton_Name(), ecorePackage.getEString(), "name", null, 0, 1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_Constraints(), theCorePackage.getConstraint(), null, "constraints", null, 0, -1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_Format(), theCorePackage.getValidatorReference(), null, "format", null, 0, 1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getRadioButton_TextProperties(), this.getTextProperty(), null, "textProperties", null, 0, -1, RadioButton.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(seperatorEClass, Seperator.class, "Seperator", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSeperator_Name(), ecorePackage.getEString(), "name", null, 0, 1, Seperator.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSeperator_Orientation(), this.getOrientation(), "orientation", null, 0, 1, Seperator.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(labelEClass, Label.class, "Label", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getLabel_Name(), ecorePackage.getEString(), "name", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLabel_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getLabel_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getLabel_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getLabel_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, Label.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(imageEClass, Image.class, "Image", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getImage_Name(), ecorePackage.getEString(), "name", null, 0, 1, Image.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getImage_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, Image.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getImage_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, Image.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(buttonEClass, Button.class, "Button", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getButton_Name(), ecorePackage.getEString(), "name", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getButton_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getButton_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getButton_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getButton_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, Button.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iCompositeEClass, IComposite.class, "IComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(xmadslCompositeEClass, XmadslComposite.class, "XmadslComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getXmadslComposite_Name(), ecorePackage.getEString(), "name", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslComposite_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getXmadslComposite_Scrollable(), theCorePackage.getBoolLiteral(), null, "scrollable", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmadslComposite_HorizontalPartitioning(), ecorePackage.getEBoolean(), "horizontalPartitioning", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmadslComposite_VerticalPartitioning(), ecorePackage.getEBoolean(), "verticalPartitioning", null, 0, 1, XmadslComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(splitPanelEClass, SplitPanel.class, "SplitPanel", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSplitPanel_SplitPosition(), ecorePackage.getEInt(), "splitPosition", null, 0, 1, SplitPanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSplitPanel_Orientation(), this.getOrientation(), "orientation", null, 0, 1, SplitPanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSplitPanel_FirstPanel(), this.getXmadslComposite(), null, "firstPanel", null, 0, 1, SplitPanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSplitPanel_SecondPanel(), this.getXmadslComposite(), null, "secondPanel", null, 0, 1, SplitPanel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedCompositeEClass, ReferencedComposite.class, "ReferencedComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedComposite_Reference(), this.getXmadslComposite(), null, "reference", null, 0, 1, ReferencedComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tableComboEClass, TableCombo.class, "TableCombo", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTableCombo_Name(), ecorePackage.getEString(), "name", null, 0, 1, TableCombo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableCombo_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, TableCombo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableCombo_Key(), theDomPackage.getPresentableFeature(), null, "key", null, 0, 1, TableCombo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableCombo_Columns(), this.getTableColumn(), null, "columns", null, 0, -1, TableCombo.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tableEClass, Table.class, "Table", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTable_Name(), ecorePackage.getEString(), "name", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Key(), theDomPackage.getPresentableFeature(), null, "key", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTable_ColumnMinWidth(), ecorePackage.getEInt(), "columnMinWidth", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_MultiselectionFlag(), this.getMultiselectionFlag(), null, "multiselectionFlag", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Oneway(), theCorePackage.getBoolLiteral(), null, "oneway", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Gridvisible(), theCorePackage.getBoolLiteral(), null, "gridvisible", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Sortindicator(), theCorePackage.getBoolLiteral(), null, "sortindicator", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Pageable(), theCorePackage.getBoolLiteral(), null, "pageable", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Filterable(), theCorePackage.getBoolLiteral(), null, "filterable", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_CheckSelection(), theCorePackage.getBoolLiteral(), null, "checkSelection", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Collapsed(), this.getCollapsedFlag(), null, "collapsed", null, 0, 1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTable_Columns(), this.getTableColumn(), null, "columns", null, 0, -1, Table.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tableColumnEClass, TableColumn.class, "TableColumn", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getTableColumn_Object(), this.getDataObjectVariable(), null, "object", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableColumn_Feature(), this.getFieldFeature(), null, "feature", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_ColumnName(), ecorePackage.getEString(), "columnName", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableColumn_Visible(), theCorePackage.getBoolLiteral(), null, "visible", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableColumn_Label(), this.getLabelProperty(), null, "label", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTableColumn_StyleProperty(), this.getStyleProperty(), null, "styleProperty", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_Align(), this.getContentAlignment(), "align", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_Width(), ecorePackage.getEInt(), "width", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_RelativeWidth(), ecorePackage.getEBoolean(), "relativeWidth", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_MinWidth(), ecorePackage.getEInt(), "minWidth", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTableColumn_MaxWidth(), ecorePackage.getEInt(), "maxWidth", null, 0, 1, TableColumn.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iTabFolderEClass, ITabFolder.class, "ITabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getITabFolder_TabItems(), this.getTabPage(), null, "tabItems", null, 0, -1, ITabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabFolderEClass, TabFolder.class, "TabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabFolder_Name(), ecorePackage.getEString(), "name", null, 0, 1, TabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedTabFolderEClass, ReferencedTabFolder.class, "ReferencedTabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedTabFolder_Reference(), this.getTabFolder(), null, "reference", null, 0, 1, ReferencedTabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iTabPageEClass, ITabPage.class, "ITabPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(tabPageEClass, TabPage.class, "TabPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabPage_Name(), ecorePackage.getEString(), "name", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_LabelText(), this.getLabelText(), null, "labelText", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_Scrollable(), theCorePackage.getBoolLiteral(), null, "scrollable", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_StyleProperty(), this.getStyleProperty(), null, "styleProperty", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_TabableFlag(), this.getTabableFlag(), null, "tabableFlag", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_PageProperties(), this.getObjectProperty(), null, "pageProperties", null, 0, -1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabPage_LogicBlock(), this.getLogicBlock(), null, "logicBlock", null, 0, 1, TabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(referencedTabPageEClass, ReferencedTabPage.class, "ReferencedTabPage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getReferencedTabPage_Reference(), this.getTabPage(), null, "reference", null, 0, 1, ReferencedTabPage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(layoutDataPropertyEClass, LayoutDataProperty.class, "LayoutDataProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(pagePropertyEClass, PageProperty.class, "PageProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(heightPropertyEClass, HeightProperty.class, "HeightProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getHeightProperty_HeightInChar(), ecorePackage.getEInt(), "heightInChar", null, 0, 1, HeightProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getHeightProperty_Negative(), ecorePackage.getEBoolean(), "negative", null, 0, 1, HeightProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getHeightProperty_Height(), ecorePackage.getEInt(), "height", null, 0, 1, HeightProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(widthPropertyEClass, WidthProperty.class, "WidthProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getWidthProperty_WidthInChar(), ecorePackage.getEInt(), "widthInChar", null, 0, 1, WidthProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getWidthProperty_Negative(), ecorePackage.getEBoolean(), "negative", null, 0, 1, WidthProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getWidthProperty_Width(), ecorePackage.getEInt(), "width", null, 0, 1, WidthProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyEClass, StyleProperty.class, "StyleProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStyleProperty_Styles(), theCorePackage.getStyle(), null, "styles", null, 0, -1, StyleProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(modalityPropertyEClass, ModalityProperty.class, "ModalityProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getModalityProperty_Modality(), ecorePackage.getEString(), "modality", null, 0, 1, ModalityProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(imagePropertyEClass, ImageProperty.class, "ImageProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getImageProperty_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, ImageProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getImageProperty_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, ImageProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(titleButtonsPropertyEClass, TitleButtonsProperty.class, "TitleButtonsProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTitleButtonsProperty_Button(), this.getTitleButton(), "button", null, 0, -1, TitleButtonsProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(resizeablePropertyEClass, ResizeableProperty.class, "ResizeableProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getResizeableProperty_Resizeable(), ecorePackage.getEBoolean(), "resizeable", null, 0, 1, ResizeableProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(statusBarPropertyEClass, StatusBarProperty.class, "StatusBarProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStatusBarProperty_StatusBar(), ecorePackage.getEBoolean(), "statusBar", null, 0, 1, StatusBarProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(centerPropertyEClass, CenterProperty.class, "CenterProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCenterProperty_Center(), ecorePackage.getEBoolean(), "center", null, 0, 1, CenterProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(alignmentPropertyEClass, AlignmentProperty.class, "AlignmentProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getAlignmentProperty_HAlignment(), this.getHorizontalAlignment(), "hAlignment", null, 0, 1, AlignmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getAlignmentProperty_VAlignment(), this.getVerticalAlignment(), "vAlignment", null, 0, 1, AlignmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(composeDataEClass, ComposeData.class, "ComposeData", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getComposeData_Tabulators(), this.getTabulatorPosition(), null, "tabulators", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_ReferencedTabulator(), this.getTabulatorDefinition(), null, "referencedTabulator", null, 0, 1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_CellWidths(), this.getTabulatorPosition(), null, "cellWidths", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getComposeData_CellWidthsCount(), ecorePackage.getEInt(), "cellWidthsCount", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_CellHeights(), this.getTabulatorPosition(), null, "cellHeights", null, 0, -1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_Spacing(), this.getPaddingWidth(), null, "spacing", null, 0, 1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getComposeData_Margin(), this.getPaddingWidth(), null, "margin", null, 0, 1, ComposeData.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(paddingWidthEClass, PaddingWidth.class, "PaddingWidth", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getPaddingWidth_Width(), ecorePackage.getEInt(), "width", null, 0, 1, PaddingWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iAttachmentPositionEClass, IAttachmentPosition.class, "IAttachmentPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getIAttachmentPosition_Name(), ecorePackage.getEString(), "name", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIAttachmentPosition_Width(), ecorePackage.getEInt(), "width", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIAttachmentPosition_WidthInChar(), ecorePackage.getEInt(), "widthInChar", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIAttachmentPosition_NegativeAfterChar(), ecorePackage.getEBoolean(), "negativeAfterChar", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIAttachmentPosition_Relative(), ecorePackage.getEBoolean(), "relative", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIAttachmentPosition_Offset(), this.getOffset(), null, "offset", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIAttachmentPosition_ReferencedPosition(), this.getIAttachmentPosition(), null, "referencedPosition", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getIAttachmentPosition_AdditionalOffset(), this.getOffset(), null, "additionalOffset", null, 0, 1, IAttachmentPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabulatorDefinitionEClass, TabulatorDefinition.class, "TabulatorDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabulatorDefinition_Name(), ecorePackage.getEString(), "name", null, 0, 1, TabulatorDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabulatorDefinition_Tabulators(), this.getTabulatorPosition(), null, "tabulators", null, 0, -1, TabulatorDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stringDefinitionEClass, StringDefinition.class, "StringDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStringDefinition_Name(), ecorePackage.getEString(), "name", null, 0, 1, StringDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStringDefinition_Text(), ecorePackage.getEString(), "text", null, 0, 1, StringDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(integerDefinitionEClass, IntegerDefinition.class, "IntegerDefinition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getIntegerDefinition_Name(), ecorePackage.getEString(), "name", null, 0, 1, IntegerDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getIntegerDefinition_Value(), ecorePackage.getEInt(), "value", null, 0, 1, IntegerDefinition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(imageUriEClass, ImageUri.class, "ImageUri", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getImageUri_Name(), ecorePackage.getEString(), "name", null, 0, 1, ImageUri.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getImageUri_Source(), ecorePackage.getEString(), "source", null, 0, 1, ImageUri.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(definitionsEClass, Definitions.class, "Definitions", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getDefinitions_Strings(), this.getStringDefinition(), null, "strings", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_Integers(), this.getIntegerDefinition(), null, "integers", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_ImageSources(), this.getImageUri(), null, "imageSources", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_Positions(), this.getAttachmentPosition(), null, "positions", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getDefinitions_Tabulators(), this.getTabulatorDefinition(), null, "tabulators", null, 0, -1, Definitions.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabulatorPositionEClass, TabulatorPosition.class, "TabulatorPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabulatorPosition_FloatingPosition(), ecorePackage.getEBoolean(), "floatingPosition", null, 0, 1, TabulatorPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTabulatorPosition_FillingPosition(), ecorePackage.getEBoolean(), "fillingPosition", null, 0, 1, TabulatorPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getTabulatorPosition_DeltaSpecification(), ecorePackage.getEBoolean(), "deltaSpecification", null, 0, 1, TabulatorPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(attachmentPositionEClass, AttachmentPosition.class, "AttachmentPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(contentEClass, Content.class, "Content", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getContent_WidgetSetItems(), this.getSetOfGuiElements(), null, "widgetSetItems", null, 0, -1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getContent_ContainerItems(), this.getComplexElement(), null, "containerItems", null, 0, -1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getContent_Elements(), this.getComposedElement(), null, "elements", null, 0, -1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getContent_ComposeLayout(), this.getComposeData(), null, "composeLayout", null, 0, 1, Content.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(setOfGuiElementsEClass, SetOfGuiElements.class, "SetOfGuiElements", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getSetOfGuiElements_FieldFlags(), this.getFieldFlag(), null, "fieldFlags", null, 0, -1, SetOfGuiElements.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSetOfGuiElements_TabableFlag(), this.getTabableFlag(), null, "tabableFlag", null, 0, 1, SetOfGuiElements.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSetOfGuiElements_Elements(), this.getComposedElement(), null, "elements", null, 0, -1, SetOfGuiElements.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(attachmentPropertyEClass, AttachmentProperty.class, "AttachmentProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getAttachmentProperty_OwnPosition(), this.getAttachmentOwnPosition(), "ownPosition", null, 0, 1, AttachmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getAttachmentProperty_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, AttachmentProperty.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(attachmentSpecificationEClass, AttachmentSpecification.class, "AttachmentSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(noneAttachmentEClass, NoneAttachment.class, "NoneAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getNoneAttachment_None(), ecorePackage.getEBoolean(), "none", null, 0, 1, NoneAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(parentAttachmentEClass, ParentAttachment.class, "ParentAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getParentAttachment_DistanceInChar(), ecorePackage.getEInt(), "distanceInChar", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getParentAttachment_Negative(), ecorePackage.getEBoolean(), "negative", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getParentAttachment_Distance(), ecorePackage.getEInt(), "distance", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getParentAttachment_Relative(), ecorePackage.getEBoolean(), "relative", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getParentAttachment_Offset(), this.getOffset(), null, "offset", null, 0, 1, ParentAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iElementOnWhichCanBeAttachedEClass, IElementOnWhichCanBeAttached.class, "IElementOnWhichCanBeAttached", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(siblingAttachmentEClass, SiblingAttachment.class, "SiblingAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSiblingAttachment_SiblingPosition(), this.getAttachmentSiblingPosition(), "siblingPosition", null, 0, 1, SiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSiblingAttachment_Offset(), this.getOffset(), null, "offset", null, 0, 1, SiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(absoluteSiblingAttachmentEClass, AbsoluteSiblingAttachment.class, "AbsoluteSiblingAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getAbsoluteSiblingAttachment_Sibling(), this.getIElementOnWhichCanBeAttached(), null, "sibling", null, 0, 1, AbsoluteSiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getAbsoluteSiblingAttachment_FieldPart(), this.getFieldPart(), "fieldPart", null, 0, 1, AbsoluteSiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(relativeSiblingAttachmentEClass, RelativeSiblingAttachment.class, "RelativeSiblingAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getRelativeSiblingAttachment_RelativeSibling(), this.getRelativeSibling(), "relativeSibling", null, 0, 1, RelativeSiblingAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tabulatorAttachmentEClass, TabulatorAttachment.class, "TabulatorAttachment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTabulatorAttachment_TabIndex(), ecorePackage.getEInt(), "tabIndex", null, 0, 1, TabulatorAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabulatorAttachment_Sibling(), this.getIElementOnWhichCanBeAttached(), null, "sibling", null, 0, 1, TabulatorAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getTabulatorAttachment_Offset(), this.getOffset(), null, "offset", null, 0, 1, TabulatorAttachment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(offsetEClass, Offset.class, "Offset", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getOffset_NegativeChar(), ecorePackage.getEBoolean(), "negativeChar", null, 0, 1, Offset.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getOffset_ValueInChar(), ecorePackage.getEInt(), "valueInChar", null, 0, 1, Offset.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getOffset_Negative(), ecorePackage.getEBoolean(), "negative", null, 0, 1, Offset.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getOffset_Value(), ecorePackage.getEInt(), "value", null, 0, 1, Offset.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(fieldVariableEClass, FieldVariable.class, "FieldVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getFieldVariable_Field(), this.getFieldReference(), null, "field", null, 0, 1, FieldVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getFieldVariable_Access(), theCorePackage.getVariableAccess(), "access", null, 0, 1, FieldVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(xmaVariableEClass, XmaVariable.class, "XmaVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getXmaVariable_Reference(), theFlexPackage.getIExpVariable(), null, "reference", null, 0, 1, XmaVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getXmaVariable_Access(), theCorePackage.getVariableAccess(), "access", null, 0, 1, XmaVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizeComponentModelEClass, CustomizeComponentModel.class, "CustomizeComponentModel", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizeComponentModel_Imports(), theCorePackage.getImport(), null, "imports", null, 0, -1, CustomizeComponentModel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeComponentModel_Component(), this.getComponent(), null, "component", null, 0, 1, CustomizeComponentModel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizeComponentModel_Customizations(), this.getCustomizationOfGuiElement(), null, "customizations", null, 0, -1, CustomizeComponentModel.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizationOfGuiElementEClass, CustomizationOfGuiElement.class, "CustomizationOfGuiElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizationOfGuiElement_Page(), this.getPage(), null, "page", null, 0, 1, CustomizationOfGuiElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizationOfCompositeEClass, CustomizationOfComposite.class, "CustomizationOfComposite", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizationOfComposite_Composite(), this.getIComposite(), null, "composite", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfComposite_ComposeLayout(), this.getComposeData(), null, "composeLayout", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfComposite_AddedElements(), this.getComposedElement(), null, "addedElements", null, 0, -1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCustomizationOfComposite_VerticalPartitioning(), ecorePackage.getEBoolean(), "verticalPartitioning", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCustomizationOfComposite_HorizontalPartitioning(), ecorePackage.getEBoolean(), "horizontalPartitioning", null, 0, 1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfComposite_AddedWidgetSetItems(), this.getSetOfGuiElements(), null, "addedWidgetSetItems", null, 0, -1, CustomizationOfComposite.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(customizationOfTabFolderEClass, CustomizationOfTabFolder.class, "CustomizationOfTabFolder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCustomizationOfTabFolder_TabFolder(), this.getITabFolder(), null, "tabFolder", null, 0, 1, CustomizationOfTabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCustomizationOfTabFolder_AddedTabItems(), this.getTabPage(), null, "addedTabItems", null, 0, -1, CustomizationOfTabFolder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(styleSpecificationEClass, StyleSpecification.class, "StyleSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStyleSpecification_ModelElements(), this.getModelElementSpecification(), null, "modelElements", null, 0, -1, StyleSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStyleSpecification_SuperStyles(), this.getStyleSpecification(), null, "superStyles", null, 0, -1, StyleSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStyleSpecification_StyleProperty(), this.getStyleSpecificationProperty(), null, "styleProperty", null, 0, -1, StyleSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(modelElementSpecificationEClass, ModelElementSpecification.class, "ModelElementSpecification", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getModelElementSpecification_ModelElement(), this.getSelectedModelElement(), null, "modelElement", null, 0, 1, ModelElementSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getModelElementSpecification_CombinedElements(), this.getCombinedModelElement(), null, "combinedElements", null, 0, -1, ModelElementSpecification.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(selectedModelElementEClass, SelectedModelElement.class, "SelectedModelElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getSelectedModelElement_Type(), this.getStyleSelectorType(), "type", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getSelectedModelElement_SimpleType(), theCorePackage.getSimpleType(), null, "simpleType", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSelectedModelElement_Name(), ecorePackage.getEString(), "name", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getSelectedModelElement_PseudoClass(), ecorePackage.getEString(), "pseudoClass", null, 0, 1, SelectedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(combinedModelElementEClass, CombinedModelElement.class, "CombinedModelElement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getCombinedModelElement_Child(), ecorePackage.getEBoolean(), "child", null, 0, 1, CombinedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCombinedModelElement_Next(), ecorePackage.getEBoolean(), "next", null, 0, 1, CombinedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCombinedModelElement_ModelElement(), this.getSelectedModelElement(), null, "modelElement", null, 0, 1, CombinedModelElement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(styleSpecificationPropertyEClass, StyleSpecificationProperty.class, "StyleSpecificationProperty", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(stylePropertyFieldPartEClass, StylePropertyFieldPart.class, "StylePropertyFieldPart", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyFieldPart_Parts(), this.getFieldPartSpecification(), null, "parts", null, 0, -1, StylePropertyFieldPart.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyLeftEClass, StylePropertyLeft.class, "StylePropertyLeft", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyLeft_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyLeft.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyRightEClass, StylePropertyRight.class, "StylePropertyRight", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyRight_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyRight.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyTopEClass, StylePropertyTop.class, "StylePropertyTop", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyTop_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyTop.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBottomEClass, StylePropertyBottom.class, "StylePropertyBottom", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyBottom_AttachmentSpecification(), this.getAttachmentSpecification(), null, "attachmentSpecification", null, 0, 1, StylePropertyBottom.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyWidthEClass, StylePropertyWidth.class, "StylePropertyWidth", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyWidth_Width(), ecorePackage.getEInt(), "width", null, 0, 1, StylePropertyWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyWidth_WidthInChar(), ecorePackage.getEInt(), "widthInChar", null, 0, 1, StylePropertyWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyWidth_NegativeAfterChar(), ecorePackage.getEBoolean(), "negativeAfterChar", null, 0, 1, StylePropertyWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyHeightEClass, StylePropertyHeight.class, "StylePropertyHeight", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyHeight_Height(), ecorePackage.getEInt(), "height", null, 0, 1, StylePropertyHeight.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyTabulatorEClass, StylePropertyTabulator.class, "StylePropertyTabulator", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyTabulator_Tabulators(), this.getTabulatorPosition(), null, "tabulators", null, 0, -1, StylePropertyTabulator.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBorderEClass, StylePropertyBorder.class, "StylePropertyBorder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyBorder_NoneBorder(), ecorePackage.getEBoolean(), "noneBorder", null, 0, 1, StylePropertyBorder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBorder_Solid(), ecorePackage.getEBoolean(), "solid", null, 0, 1, StylePropertyBorder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyForegroundColorEClass, StylePropertyForegroundColor.class, "StylePropertyForegroundColor", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyForegroundColor_Red(), ecorePackage.getEInt(), "red", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyForegroundColor_Green(), ecorePackage.getEInt(), "green", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyForegroundColor_Blue(), ecorePackage.getEInt(), "blue", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyForegroundColor_ColorID(), ecorePackage.getEString(), "colorID", null, 0, 1, StylePropertyForegroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBackgroundColorEClass, StylePropertyBackgroundColor.class, "StylePropertyBackgroundColor", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyBackgroundColor_Red(), ecorePackage.getEInt(), "red", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBackgroundColor_Green(), ecorePackage.getEInt(), "green", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBackgroundColor_Blue(), ecorePackage.getEInt(), "blue", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyBackgroundColor_ColorID(), ecorePackage.getEString(), "colorID", null, 0, 1, StylePropertyBackgroundColor.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyWidgetVariantEClass, StylePropertyWidgetVariant.class, "StylePropertyWidgetVariant", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyWidgetVariant_Name(), ecorePackage.getEString(), "name", null, 0, 1, StylePropertyWidgetVariant.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyImageEClass, StylePropertyImage.class, "StylePropertyImage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyImage_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, StylePropertyImage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyImage_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, StylePropertyImage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyAlignmentEClass, StylePropertyAlignment.class, "StylePropertyAlignment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyAlignment_Alignment(), this.getContentAlignment(), "alignment", null, 0, 1, StylePropertyAlignment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyTableEClass, StylePropertyTable.class, "StylePropertyTable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyTable_OneWay(), theCorePackage.getBoolLiteral(), null, "oneWay", null, 0, 1, StylePropertyTable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyTable_GridVisible(), theCorePackage.getBoolLiteral(), null, "gridVisible", null, 0, 1, StylePropertyTable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyFontEClass, StylePropertyFont.class, "StylePropertyFont", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyFont_Styles(), ecorePackage.getEString(), "styles", null, 0, -1, StylePropertyFont.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyFont_Size(), ecorePackage.getEInt(), "size", null, 0, 1, StylePropertyFont.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyFont_Name(), ecorePackage.getEString(), "name", null, 0, -1, StylePropertyFont.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyCharacterWidthEClass, StylePropertyCharacterWidth.class, "StylePropertyCharacterWidth", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyCharacterWidth_CharWidth(), ecorePackage.getEInt(), "charWidth", null, 0, 1, StylePropertyCharacterWidth.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyCharacterHeightEClass, StylePropertyCharacterHeight.class, "StylePropertyCharacterHeight", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyCharacterHeight_CharHeight(), ecorePackage.getEInt(), "charHeight", null, 0, 1, StylePropertyCharacterHeight.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyFlagEClass, StylePropertyFlag.class, "StylePropertyFlag", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyFlag_EnabledExpression(), theCorePackage.getEqualityExpr(), null, "enabledExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyFlag_MandatoryExpression(), theCorePackage.getEqualityExpr(), null, "mandatoryExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyFlag_VisibleExpression(), theCorePackage.getEqualityExpr(), null, "visibleExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyFlag_CollapsedExpression(), theCorePackage.getEqualityExpr(), null, "CollapsedExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyFlag_AvailableExpression(), theCorePackage.getEqualityExpr(), null, "availableExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyFlag_EditableExpression(), theCorePackage.getEqualityExpr(), null, "editableExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyFlag_TabableExpression(), theCorePackage.getEqualityExpr(), null, "tabableExpression", null, 0, 1, StylePropertyFlag.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyLineWrapEClass, StylePropertyLineWrap.class, "StylePropertyLineWrap", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyLineWrap_Wrap(), theCorePackage.getBoolLiteral(), null, "wrap", null, 0, 1, StylePropertyLineWrap.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyMultiLineEClass, StylePropertyMultiLine.class, "StylePropertyMultiLine", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyMultiLine_Multi(), theCorePackage.getBoolLiteral(), null, "multi", null, 0, 1, StylePropertyMultiLine.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyBrowseControlEClass, StylePropertyBrowseControl.class, "StylePropertyBrowseControl", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyBrowseControl_ControlType(), this.getBrowseControlType(), "controlType", null, 0, 1, StylePropertyBrowseControl.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyPagingEClass, StylePropertyPaging.class, "StylePropertyPaging", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyPaging_Style(), ecorePackage.getEString(), "style", null, 0, 1, StylePropertyPaging.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyPaging_Default(), ecorePackage.getEBoolean(), "default", null, 0, 1, StylePropertyPaging.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyPagingControlsEClass, StylePropertyPagingControls.class, "StylePropertyPagingControls", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyPagingControls_Controls(), this.getPagingControl(), null, "controls", null, 0, -1, StylePropertyPagingControls.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyPagingPageSizeEClass, StylePropertyPagingPageSize.class, "StylePropertyPagingPageSize", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyPagingPageSize_Size(), ecorePackage.getEInt(), "size", null, 0, 1, StylePropertyPagingPageSize.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyPagingPageSize_MaxSize(), ecorePackage.getEInt(), "maxSize", null, 0, 1, StylePropertyPagingPageSize.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyPagingJumpSizeEClass, StylePropertyPagingJumpSize.class, "StylePropertyPagingJumpSize", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyPagingJumpSize_Size(), ecorePackage.getEInt(), "size", null, 0, 1, StylePropertyPagingJumpSize.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyPagingPositionEClass, StylePropertyPagingPosition.class, "StylePropertyPagingPosition", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyPagingPosition_Top(), ecorePackage.getEBoolean(), "top", null, 0, 1, StylePropertyPagingPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyPagingPosition_Bottom(), ecorePackage.getEBoolean(), "bottom", null, 0, 1, StylePropertyPagingPosition.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyPagingCustomizerImageEClass, StylePropertyPagingCustomizerImage.class, "StylePropertyPagingCustomizerImage", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyPagingCustomizerImage_ImageUri(), ecorePackage.getEString(), "imageUri", null, 0, 1, StylePropertyPagingCustomizerImage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyPagingCustomizerImage_ImageUriReference(), this.getImageUri(), null, "imageUriReference", null, 0, 1, StylePropertyPagingCustomizerImage.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(pagingControlEClass, PagingControl.class, "PagingControl", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getPagingControl_Control(), ecorePackage.getEString(), "control", null, 0, 1, PagingControl.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPagingControl_End(), ecorePackage.getEBoolean(), "end", null, 0, 1, PagingControl.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPagingControl_Next(), ecorePackage.getEBoolean(), "next", null, 0, 1, PagingControl.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPagingControl_Customize(), ecorePackage.getEBoolean(), "customize", null, 0, 1, PagingControl.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPagingControl_Export(), ecorePackage.getEBoolean(), "export", null, 0, 1, PagingControl.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyTableCustomizerEClass, StylePropertyTableCustomizer.class, "StylePropertyTableCustomizer", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyTableCustomizer_TableCustomizer(), this.getTableCustomizer(), null, "tableCustomizer", null, 0, 1, StylePropertyTableCustomizer.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyScrollableEClass, StylePropertyScrollable.class, "StylePropertyScrollable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyScrollable_NotScrollable(), ecorePackage.getEBoolean(), "notScrollable", null, 0, 1, StylePropertyScrollable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyScrollable_Orientations(), this.getOrientation(), "orientations", null, 0, -1, StylePropertyScrollable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyEnumerationDisplayTypeEClass, StylePropertyEnumerationDisplayType.class, "StylePropertyEnumerationDisplayType", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyEnumerationDisplayType_Type(), ecorePackage.getEString(), "type", null, 0, 1, StylePropertyEnumerationDisplayType.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyEnumerationSortOrderEClass, StylePropertyEnumerationSortOrder.class, "StylePropertyEnumerationSortOrder", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getStylePropertyEnumerationSortOrder_Sortorder(), ecorePackage.getEString(), "sortorder", null, 0, 1, StylePropertyEnumerationSortOrder.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(stylePropertyOtherEClass, StylePropertyOther.class, "StylePropertyOther", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStylePropertyOther_DynamicTabPage(), theCorePackage.getBoolLiteral(), null, "dynamicTabPage", null, 0, 1, StylePropertyOther.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyOther_ThousandSeperator(), theCorePackage.getBoolLiteral(), null, "thousandSeperator", null, 0, 1, StylePropertyOther.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyOther_Spacing(), this.getPaddingWidth(), null, "spacing", null, 0, 1, StylePropertyOther.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getStylePropertyOther_Margin(), this.getPaddingWidth(), null, "margin", null, 0, 1, StylePropertyOther.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getStylePropertyOther_WidgetEffects(), ecorePackage.getEString(), "widgetEffects", null, 0, -1, StylePropertyOther.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, !IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(tableCustomizerEClass, TableCustomizer.class, "TableCustomizer", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getTableCustomizer_InstanceType(), ecorePackage.getEString(), "instanceType", null, 0, 1, TableCustomizer.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(commandImplementationEClass, CommandImplementation.class, "CommandImplementation", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCommandImplementation_Statements(), this.getStatement(), null, "statements", null, 0, -1, CommandImplementation.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(statementEClass, Statement.class, "Statement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(statementVariableEClass, StatementVariable.class, "StatementVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getStatementVariable_Reference(), this.getIReferenceableByStatementVariable(), null, "reference", null, 0, 1, StatementVariable.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(iReferenceableByStatementVariableEClass, IReferenceableByStatementVariable.class, "IReferenceableByStatementVariable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(assignStatementEClass, AssignStatement.class, "AssignStatement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getAssignStatement_VariableAssignments(), this.getVariableAssignment(), null, "variableAssignments", null, 0, 1, AssignStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(variableAssignmentEClass, VariableAssignment.class, "VariableAssignment", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getVariableAssignment_Left(), this.getStatementVariable(), null, "left", null, 0, 1, VariableAssignment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getVariableAssignment_Value(), this.getVariableValue(), null, "value", null, 0, 1, VariableAssignment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getVariableAssignment_ServiceCall(), this.getExecuteStatement(), null, "serviceCall", null, 0, 1, VariableAssignment.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(variableValueEClass, VariableValue.class, "VariableValue", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getVariableValue_Variable(), this.getStatementVariable(), null, "variable", null, 0, 1, VariableValue.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getVariableValue_Access(), theCorePackage.getVariableAccess(), "access", null, 0, 1, VariableValue.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getVariableValue_StringLiteral(), ecorePackage.getEString(), "stringLiteral", null, 0, 1, VariableValue.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getVariableValue_IntegerLiteral(), ecorePackage.getEInt(), "integerLiteral", null, 0, 1, VariableValue.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(executeStatementEClass, ExecuteStatement.class, "ExecuteStatement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getExecuteStatement_Service(), theDomPackage.getService(), null, "service", null, 0, 1, ExecuteStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getExecuteStatement_Operation(), ecorePackage.getEObject(), null, "operation", null, 0, 1, ExecuteStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getExecuteStatement_DelegateOperation(), theDomPackage.getDelegateOperation(), null, "delegateOperation", null, 0, 1, ExecuteStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getExecuteStatement_ParameterValues(), this.getVariableValue(), null, "parameterValues", null, 0, -1, ExecuteStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(invokeStatementEClass, InvokeStatement.class, "InvokeStatement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getInvokeStatement_InvokeAble(), this.getInvokeable(), null, "invokeAble", null, 0, 1, InvokeStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getInvokeStatement_TargetPanel(), this.getXmadslComposite(), null, "targetPanel", null, 0, 1, InvokeStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getInvokeStatement_PropertyJoin(), this.getPropertyJoin(), null, "propertyJoin", null, 0, -1, InvokeStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(callStatementEClass, CallStatement.class, "CallStatement", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getCallStatement_Command(), this.getCommand(), null, "command", null, 0, 1, CallStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCallStatement_Left(), this.getStatementVariable(), null, "left", null, 0, 1, CallStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getCallStatement_JavaCommand(), ecorePackage.getEString(), "javaCommand", null, 0, 1, CallStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getCallStatement_ParameterValues(), this.getVariableValue(), null, "parameterValues", null, 0, -1, CallStatement.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(invokeableEClass, Invokeable.class, "Invokeable", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);

		initEClass(proxyEClass, Proxy.class, "Proxy", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEAttribute(getProxy_Url(), ecorePackage.getEString(), "url", null, 0, 1, Proxy.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getProxy_Properties(), this.getObjectProperty(), null, "properties", null, 0, -1, Proxy.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		initEClass(propertyJoinEClass, PropertyJoin.class, "PropertyJoin", !IS_ABSTRACT, !IS_INTERFACE, IS_GENERATED_INSTANCE_CLASS);
		initEReference(getPropertyJoin_Variable(), this.getStatementVariable(), null, "variable", null, 0, 1, PropertyJoin.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, IS_COMPOSITE, !IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPropertyJoin_Access(), theCorePackage.getVariableAccess(), "access", null, 0, 1, PropertyJoin.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPropertyJoin_StringLiteral(), ecorePackage.getEString(), "stringLiteral", null, 0, 1, PropertyJoin.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPropertyJoin_IntegerLiteral(), ecorePackage.getEInt(), "integerLiteral", null, 0, 1, PropertyJoin.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEAttribute(getPropertyJoin_JoinDirection(), this.getJoinDirection(), "joinDirection", null, 0, 1, PropertyJoin.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_UNSETTABLE, !IS_ID, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);
		initEReference(getPropertyJoin_Property(), this.getObjectProperty(), null, "property", null, 0, 1, PropertyJoin.class, !IS_TRANSIENT, !IS_VOLATILE, IS_CHANGEABLE, !IS_COMPOSITE, IS_RESOLVE_PROXIES, !IS_UNSETTABLE, IS_UNIQUE, !IS_DERIVED, IS_ORDERED);

		// Initialize enums and add enum literals
		initEEnum(eventTypeEEnum, EventType.class, "EventType");
		addEEnumLiteral(eventTypeEEnum, EventType.NULL);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_CLICK);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_DOUBLE_CLICK);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_SELECTION);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_DEFAULT_SELECTION);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_EXPAND);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_COLLAPSE);
		addEEnumLiteral(eventTypeEEnum, EventType.ON_LOAD);

		initEEnum(predefinedCommandEEnum, PredefinedCommand.class, "PredefinedCommand");
		addEEnumLiteral(predefinedCommandEEnum, PredefinedCommand.CLOSE_OK);
		addEEnumLiteral(predefinedCommandEEnum, PredefinedCommand.CLOSE_CANCEL);

		initEEnum(attachmentOwnPositionEEnum, AttachmentOwnPosition.class, "AttachmentOwnPosition");
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.LEFT);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.RIGHT);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.TOP);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.BOTTOM);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.VCENTER);
		addEEnumLiteral(attachmentOwnPositionEEnum, AttachmentOwnPosition.HCENTER);

		initEEnum(relativeSiblingEEnum, RelativeSibling.class, "RelativeSibling");
		addEEnumLiteral(relativeSiblingEEnum, RelativeSibling.PREVIOUS);
		addEEnumLiteral(relativeSiblingEEnum, RelativeSibling.NEXT);

		initEEnum(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.class, "AttachmentSiblingPosition");
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.NULL);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.LEFT);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.RIGHT);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.TOP);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.BOTTOM);
		addEEnumLiteral(attachmentSiblingPositionEEnum, AttachmentSiblingPosition.CENTER);

		initEEnum(contentAlignmentEEnum, ContentAlignment.class, "ContentAlignment");
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.DEFAULT);
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.LEFT);
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.RIGHT);
		addEEnumLiteral(contentAlignmentEEnum, ContentAlignment.CENTER);

		initEEnum(tabAlignmentEEnum, TabAlignment.class, "TabAlignment");
		addEEnumLiteral(tabAlignmentEEnum, TabAlignment.DEFAULT);
		addEEnumLiteral(tabAlignmentEEnum, TabAlignment.LEFT);
		addEEnumLiteral(tabAlignmentEEnum, TabAlignment.RIGHT);

		initEEnum(horizontalAlignmentEEnum, HorizontalAlignment.class, "HorizontalAlignment");
		addEEnumLiteral(horizontalAlignmentEEnum, HorizontalAlignment.LEFT);
		addEEnumLiteral(horizontalAlignmentEEnum, HorizontalAlignment.RIGHT);

		initEEnum(verticalAlignmentEEnum, VerticalAlignment.class, "VerticalAlignment");
		addEEnumLiteral(verticalAlignmentEEnum, VerticalAlignment.TOP);
		addEEnumLiteral(verticalAlignmentEEnum, VerticalAlignment.BOTTOM);

		initEEnum(orientationEEnum, Orientation.class, "Orientation");
		addEEnumLiteral(orientationEEnum, Orientation.HORIZONTAL);
		addEEnumLiteral(orientationEEnum, Orientation.VERTICAL);

		initEEnum(controlTypeEEnum, ControlType.class, "ControlType");
		addEEnumLiteral(controlTypeEEnum, ControlType.TEXT);
		addEEnumLiteral(controlTypeEEnum, ControlType.TABLECOMBO);
		addEEnumLiteral(controlTypeEEnum, ControlType.COMBO);
		addEEnumLiteral(controlTypeEEnum, ControlType.CHECKBOX);
		addEEnumLiteral(controlTypeEEnum, ControlType.LISTBOX);
		addEEnumLiteral(controlTypeEEnum, ControlType.LABEL);

		initEEnum(fieldPartEEnum, FieldPart.class, "FieldPart");
		addEEnumLiteral(fieldPartEEnum, FieldPart.LABEL);
		addEEnumLiteral(fieldPartEEnum, FieldPart.CONTROL);
		addEEnumLiteral(fieldPartEEnum, FieldPart.UNITLABEL);
		addEEnumLiteral(fieldPartEEnum, FieldPart.EDITOR);

		initEEnum(titleButtonEEnum, TitleButton.class, "TitleButton");
		addEEnumLiteral(titleButtonEEnum, TitleButton.MINIMIZE);
		addEEnumLiteral(titleButtonEEnum, TitleButton.MAXIMIZE);
		addEEnumLiteral(titleButtonEEnum, TitleButton.CLOSE);

		initEEnum(joinDirectionEEnum, JoinDirection.class, "JoinDirection");
		addEEnumLiteral(joinDirectionEEnum, JoinDirection.IN);
		addEEnumLiteral(joinDirectionEEnum, JoinDirection.OUT);
		addEEnumLiteral(joinDirectionEEnum, JoinDirection.INOUT);

		initEEnum(styleSelectorTypeEEnum, StyleSelectorType.class, "StyleSelectorType");
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.ANY);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.LABEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.FIELD);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.CELL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TEXT);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.DATEPICKER);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.COMBO);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABLE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABLECOMBO);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.COLUMN);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TREE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.CHECKBOX);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.RADIOBUTTON);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.LISTBOX);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.PANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.VPANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.HPANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.IPANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.SPLITPANEL);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABFOLDER);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.TABPAGE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.IMAGE);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.SEPERATOR);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.BUTTON);
		addEEnumLiteral(styleSelectorTypeEEnum, StyleSelectorType.PAGE);

		initEEnum(browseControlTypeEEnum, BrowseControlType.class, "BrowseControlType");
		addEEnumLiteral(browseControlTypeEEnum, BrowseControlType.AUTO);
		addEEnumLiteral(browseControlTypeEEnum, BrowseControlType.LABEL);
		addEEnumLiteral(browseControlTypeEEnum, BrowseControlType.TEXT);

		// Create resource
		createResource(eNS_URI);
	}

} //PomPackageImpl
